@@ -12,18 +12,21 @@ int CodeSnippetPlaceHolder::offsetEnd(){
 }
 
 
-inline void translatePlaceholder(const QString& content, QString& curLine, CodeSnippetPlaceHolder& ph){
+inline void translatePlaceholder(const QString& content, QString& curLine, CodeSnippetPlaceHolder& ph, int columnshift = 0){
+	bool translatable = (ph.flags & CodeSnippetPlaceHolder::Translatable) && !CodeSnippet::debugDisableAutoTranslate;
+	if (translatable)
 	for (int i=0;i<content.length();i++) 
 		if (!content[i].toAscii()){ //don't translate non ascii placeholders
-			curLine+=content;
-			ph.length+=content.length();
-			return;
+			translatable = false;
+			break;
 		}
-			
-	QString trans = QApplication::translate("CodeSnippet_PlaceHolder", content.toAscii().constData());
-	curLine += trans;
+	QString trans = !translatable ? content : QApplication::translate("CodeSnippet_PlaceHolder", content.toAscii().constData());
+	if (columnshift < 0) {
+		curLine = curLine.left(curLine.length() + columnshift) + trans + curLine.right(-columnshift); 
+		ph.offset += columnshift;
+		//todo: allow positive shift
+	} else curLine += trans;
 	ph.length = trans.length();
-	return;
 }
 
 
@@ -48,26 +51,30 @@ void parseSnippetPlaceHolder(const QString& snippet, int& i, QString& curLine, C
 	
 	
 	secondLevelBreak:
-	translatePlaceholder(tmpPlaceHolderContent, curLine, ph);
-	if (i>=snippet.length()) return;
-	if (snippet.at(i)!=':') return;
+	int snippetEnd;
+	if ((snippet.at(i)!=':') || ((snippetEnd = snippet.indexOf("%>", i)) == -1)) return;
 
-	int snippetEnd = snippet.indexOf("%>", i);
-	if (snippetEnd == -1) return;
 	QString options = snippet.mid(i+1, snippetEnd-i-1);
 	i = snippetEnd+1;
+	int columnshift = 0;
 	foreach (const QString& s, options.split(",")) {
 		QString t = s.trimmed();
 		if (t == "mirror") ph.flags|=CodeSnippetPlaceHolder::Mirror;
 		else if (t == "multiline") ph.flags|=CodeSnippetPlaceHolder::PreferredMultilineAutoSelect;
 		else if (t.startsWith("id:")) ph.id = t.remove(0,3).toInt();
+		else if (t.startsWith("columnShift:", Qt::CaseInsensitive)) columnshift = t.remove(0,12).toInt();
 		else if (t.startsWith("select")) ph.flags|=CodeSnippetPlaceHolder::AutoSelect;
+		else if (t == "persistent") ph.flags|=CodeSnippetPlaceHolder::Persistent;
+		else if (t == "translatable") ph.flags|=CodeSnippetPlaceHolder::Translatable;
 	}
+
+	translatePlaceholder(tmpPlaceHolderContent, curLine, ph, columnshift);
 }
 
 bool CodeSnippet::autoReplaceCommands=true;
+bool CodeSnippet::debugDisableAutoTranslate = false;
 
-CodeSnippet::CodeSnippet(const QString &newWord) {
+CodeSnippet::CodeSnippet(const QString &newWord, bool replacePercentNewline) {
 	QString realNewWord=newWord;
 	// \begin magic
 	if (newWord == "%<%:TEXMAKERX-GENERIC-ENVIRONMENT-TEMPLATE%>" ||
@@ -153,12 +160,18 @@ CodeSnippet::CodeSnippet(const QString &newWord) {
 				placeHolders.last().append(tempPlaceholder);
 			//	foundDescription = true;
 				break;
+			case '\n':
+				curLine += "%"; word += "%";
+				// no break
 			case 'n':
-				lines.append(curLine);
-				placeHolders.append(QList<CodeSnippetPlaceHolder>());
-				curLine.clear();
-				//curLine+="\n";
-				break;	
+				if (currentChar.toAscii() == '\n' || replacePercentNewline) {
+					lines.append(curLine);
+					placeHolders.append(QList<CodeSnippetPlaceHolder>());
+					curLine.clear();
+					firstLine = false;
+					//curLine+="\n";
+					break;
+				}
 			default: // escape was not an escape character ...
 				curLine+='%';
 				curLine+=currentChar;
@@ -254,7 +267,7 @@ void CodeSnippet::insertAt(QEditor* editor, QDocumentCursor* cursor, bool usePla
 	QDocumentLine curLine=cursor->line();
 
 	// on multi line commands, replace environments only
-	if(autoReplaceCommands && lines.size()>1 && line.contains("\\begin{")){
+    if(autoReplaceCommands && byCompleter && lines.size()>1 && line.contains("\\begin{")){
 		QString curLine=cursor->line().text();
 		int wordBreak=curLine.indexOf(QRegExp("\\W"),cursor->columnNumber());
 		int closeCurl=curLine.indexOf("}",cursor->columnNumber());
@@ -301,6 +314,7 @@ void CodeSnippet::insertAt(QEditor* editor, QDocumentCursor* cursor, bool usePla
 				pos=first.indexOf('{',pos+1); //pos of second {
 				if(pos>-1)
 				    first.remove(pos,first.length()-pos);
+				editor->document()->clearLanguageMatches();
 				editor->insertText(*cursor,first);
 				if (editBlockOpened) cursor->endEditBlock();
 				return;
@@ -311,12 +325,13 @@ void CodeSnippet::insertAt(QEditor* editor, QDocumentCursor* cursor, bool usePla
 	int baseLine=cursor->lineNumber();
 	int baseLineIndent = cursor->columnNumber(); //text before inserted word moves placeholders to the right
 	int lastLineRemainingLength = curLine.text().length()-baseLineIndent; //last line will has length: indentation + codesnippet + lastLineRemainingLength
+	editor->document()->clearLanguageMatches();
 	editor->insertText(*cursor,line); //don't use cursor->insertText to keep autoindentation working
 
 	if (editBlockOpened) cursor->endEditBlock();
 
 	// on single line commands only: replace command
-	if(byCompleter && autoReplaceCommands && lines.size()==1 && line.startsWith('\\')){
+    if(byCompleter && autoReplaceCommands && lines.size()==1 && line.startsWith('\\')){
 		if(cursor->nextChar().isLetterOrNumber()||cursor->nextChar()==QChar('{')){
 			QString curLine=cursor->line().text();
 			int wordBreak=curLine.indexOf(QRegExp("\\W"),cursor->columnNumber());
@@ -360,6 +375,7 @@ void CodeSnippet::insertAt(QEditor* editor, QDocumentCursor* cursor, bool usePla
 		for (int l=0;l< lines.count();l++)
 			usePlaceholders|=placeHolders[l].size();
 	}
+	
 	int autoSelectPlaceholder = -1;
 	if (usePlaceholders) {
 		if (editor->currentPlaceHolder()!=-1 && 
@@ -372,6 +388,7 @@ void CodeSnippet::insertAt(QEditor* editor, QDocumentCursor* cursor, bool usePla
 				PlaceHolder ph;
 				ph.length=placeHolders[l][i].length;
 				ph.cursor = getCursor(editor, placeHolders[l][i], l, baseLine, baseLineIndent, lastLineRemainingLength);
+				ph.autoRemove = !(placeHolders[l][i].flags & CodeSnippetPlaceHolder::Persistent);
 				if (!ph.cursor.isValid()) continue;
 				editor->addPlaceHolder(ph);
 				if (placeHolders[l][i].flags & CodeSnippetPlaceHolder::Mirrored) {
