@@ -11,18 +11,22 @@
 
 #include "configdialog.h"
 
+#include "updatechecker.h"
+
 #include "qdocument.h"
 
 #include "spellerutility.h"
 #include "latexeditorview_config.h"
 #include "smallUsefulFunctions.h"
-#include "buildmanager.h"
 #include "qformatconfig.h"
 
 const QString ShortcutDelegate::addRowButton="<internal: add row>";
 const QString ShortcutDelegate::deleteRowButton="<internal: delete row>";
 
+static const QString nameSeparator = "separator";
+
 ShortcutComboBox::ShortcutComboBox(QWidget *parent):QComboBox(parent){
+	setMaxVisibleItems(15);
 	addItem(tr("<default>"));
 	addItem(tr("<none>"));
 	for (int k=Qt::Key_F1; k<=Qt::Key_F12; k++)
@@ -30,25 +34,32 @@ ShortcutComboBox::ShortcutComboBox(QWidget *parent):QComboBox(parent){
 	for (int c=0; c<=1; c++)
 		for (int s=0; s<=1; s++)
 			for (int a=0; a<=1; a++) {
-		if (!c && !s && !a) continue;
-		for (int k=Qt::Key_F1; k<=Qt::Key_F12; k++)
-			addItem(QKeySequence(c*Qt::CTRL+s*Qt::SHIFT+a*Qt::ALT+k).toString(QKeySequence::NativeText));
-		for (int k=Qt::Key_0; k<=Qt::Key_9; k++)
-			addItem(QKeySequence(c*Qt::CTRL+s*Qt::SHIFT+a*Qt::ALT+k).toString(QKeySequence::NativeText));
-		for (int k=Qt::Key_A; k<=Qt::Key_Z; k++)
-			addItem(QKeySequence(c*Qt::CTRL+s*Qt::SHIFT+a*Qt::ALT+k).toString(QKeySequence::NativeText));
-		if (a || (c&&s)){
-			for (int k=Qt::Key_PageUp; k<=Qt::Key_PageDown; k++)
-				addItem(QKeySequence(c*Qt::CTRL+s*Qt::SHIFT+a*Qt::ALT+k).toString(QKeySequence::NativeText));
-		}
-	}
+				if (a || c || s) {
+					for (int k=Qt::Key_F1; k<=Qt::Key_F12; k++)
+						addItem(QKeySequence(c*Qt::CTRL+s*Qt::SHIFT+a*Qt::ALT+k).toString(QKeySequence::NativeText));
+					for (int k=Qt::Key_0; k<=Qt::Key_9; k++)
+						addItem(QKeySequence(c*Qt::CTRL+s*Qt::SHIFT+a*Qt::ALT+k).toString(QKeySequence::NativeText));
+					for (int k=Qt::Key_A; k<=Qt::Key_Z; k++)
+						addItem(QKeySequence(c*Qt::CTRL+s*Qt::SHIFT+a*Qt::ALT+k).toString(QKeySequence::NativeText));
+					if (a || c) {
+						for (int k=Qt::Key_Left; k<=Qt::Key_Down; k++)
+							addItem(QKeySequence(c*Qt::CTRL+s*Qt::SHIFT+a*Qt::ALT+k).toString(QKeySequence::NativeText));
+						for (int k=Qt::Key_PageUp; k<=Qt::Key_PageDown; k++)
+							addItem(QKeySequence(c*Qt::CTRL+s*Qt::SHIFT+a*Qt::ALT+k).toString(QKeySequence::NativeText));
+					}
+				}
+				addItem(QKeySequence(c*Qt::CTRL+s*Qt::SHIFT+a*Qt::ALT+Qt::Key_Tab).toString(QKeySequence::NativeText));
+			}
 	setEditable(true);
 }
 
 void ShortcutComboBox::keyPressEvent(QKeyEvent *e){
-	if ( (e->modifiers()!=0 && e->key() != Qt::Key_Alt && e->key() != Qt::Key_Shift && e->key() != Qt::Key_Control && e->key() != Qt::Key_AltGr && e->key() != Qt::Key_Meta && e->key() != 0 && e->key() != Qt::Key_Super_L && e->key() != Qt::Key_Super_R)
+	if ( (e->modifiers()!=0 && e->text() != "+" && e->key() != Qt::Key_Alt && e->key() != Qt::Key_Shift && e->key() != Qt::Key_Control && e->key() != Qt::Key_AltGr && e->key() != Qt::Key_Meta && e->key() != 0 && e->key() != Qt::Key_Super_L && e->key() != Qt::Key_Super_R)
 		|| (e->key() >= Qt::Key_F1 &&  e->key() <= Qt::Key_F35)) {
-		QString newShortCut = QKeySequence(e->modifiers() | e->key()).toString(QKeySequence::NativeText);
+		// FIXME: Qt currently does not handle KeypadModifier correctly.
+		// as a workaround we just take it away, so there is no difference between keypad and non-keypad keys,
+		// but at least keypad keys don't produce rubbish. See also sf.net bug item 3525266
+		QString newShortCut = QKeySequence((e->modifiers() | e->key()) & ~Qt::KeypadModifier).toString(QKeySequence::NativeText);
 		int index = findText(newShortCut);
 		if (index != -1) setCurrentIndex(index);
 		else setEditText(newShortCut);
@@ -57,16 +68,19 @@ void ShortcutComboBox::keyPressEvent(QKeyEvent *e){
 	QComboBox::keyPressEvent(e);
 }
 
+void ShortcutComboBox::focusInEvent(QFocusEvent *e) {
+	Q_UNUSED(e);
+	this->lineEdit()->selectAll();
+}
+
 
 ShortcutDelegate::ShortcutDelegate(QObject *parent): treeWidget(0) {
-	// remove unused argument warning
-	(void) parent;
+	Q_UNUSED(parent);
 }
 QWidget *ShortcutDelegate::createEditor(QWidget *parent,
                                         const QStyleOptionViewItem & option ,
                                         const QModelIndex & index) const {
-	// remove unused argument warning
-	(void) option;
+	Q_UNUSED(option);
 
 	if (!index.isValid()) return 0;
 	const QAbstractItemModel *model = index.model();
@@ -91,8 +105,10 @@ QWidget *ShortcutDelegate::createEditor(QWidget *parent,
 	}
 
 	//menu shortcut key
-	if (index.column()==1) txsWarning(tr("Sorry, you clicked in the wrong column.\nTo change a shortcut, you have to edit the third or fourth column."));
-	if (index.column()!=2 && index.column()!=3) return 0;
+	if (index.column()!=2 && index.column()!=3) {
+		txsWarning(tr("To change a shortcut, edit the column \"Current Shortcut\" or \"Additional Shortcut\"."));
+		return 0;
+	}
 	ShortcutComboBox *editor = new ShortcutComboBox(parent);
 
 	return editor;
@@ -113,6 +129,7 @@ void ShortcutDelegate::setEditorData(QWidget *editor,
 		int pos=box->findText(normalized);
 		if (pos==-1) box->setEditText(value);
 		else box->setCurrentIndex(pos);
+		if (box->lineEdit()) box->lineEdit()->selectAll();
 		return;
 	}
 	//editor key replacement
@@ -154,6 +171,14 @@ void ShortcutDelegate::setModelData(QWidget *editor, QAbstractItemModel *model,
 			txsWarning(ConfigDialog::tr("The shortcut you entered is invalid."));
 			return;
 		}
+		QRegExp rxCharKey("(Shift\\+)?."); // matches all single characters and single characters with shift like "Shift+A", should not match e.g. "F1" or "DEL"
+		if (rxCharKey.exactMatch(value)) {
+			if (!txsConfirmWarning(ConfigDialog::tr("The shortcut you entered is a standard character key.\n"
+													"You will not be able to type this character. Do you wish\n"
+													"to set the key anyway?"))) {
+				return;
+			}
+		}
 
 		/*int r=-1;
 		for (int i=0;i<model->rowCount();i++)
@@ -173,10 +198,12 @@ void ShortcutDelegate::setModelData(QWidget *editor, QAbstractItemModel *model,
 			REQUIRE(editorKeys);
 			if (!li.empty() && li.first() && (li.first()->parent() != editorKeys || isBasicEditorKey(index))) {
 				QString duplicate=li.first()->text(0);//model->data(model->index(mil[0].row(),0,mil[0].parent()),Qt::DisplayRole).toString();
-				if (txsConfirmWarning(ConfigDialog::tr("The shortcut you entered is the same as the one of this command:") +"\n"+duplicate+"\n"+ConfigDialog::tr("Should I delete this other shortcut?"))) {
+				if (txsConfirmWarning(QString(ConfigDialog::tr("The shortcut <%1> is already assigned to the command:")).arg(value) +"\n"+duplicate+"\n\n"+ConfigDialog::tr("Do you wish to remove the old assignment and bind the shortcut to the new command?"))) {
 					//model->setData(mil[0],"",Qt::DisplayRole);
 					if (li[0]->text(2) == value)  li[0]->setText(2,"");
 					else if (li[0]->text(3) == value)  li[0]->setText(3,"");
+				} else {
+					return;
 				}
 			}
 		}
@@ -247,9 +274,54 @@ bool ShortcutDelegate::isBasicEditorKey(const QModelIndex& index) const{
 }
 
 
+
+ComboBoxDelegate::ComboBoxDelegate(QObject *parent): QItemDelegate(parent){
+	activeColumn = 2;
+}
+QWidget *ComboBoxDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option,const QModelIndex &index) const{
+	if (index.column() != activeColumn) 
+		return QItemDelegate::createEditor(parent,option,index);
+	QComboBox *editor = new QComboBox(parent);
+	editor->addItems(defaultItems);
+	editor->setEditable(true);
+	
+	return editor;
+}
+void ComboBoxDelegate::setEditorData(QWidget *editor, const QModelIndex &index) const{
+	if (index.column() != activeColumn) {
+		QItemDelegate::setEditorData(editor, index);
+		return;
+	}
+	QComboBox* cb = qobject_cast<QComboBox*>(editor);
+	REQUIRE(cb);
+	QString s = index.data(Qt::EditRole).toString();
+	if (s.contains('(')) s = s.left(s.indexOf('('));
+	for (int i=0;i<cb->count();i++)
+		if (cb->itemText(i).startsWith(s)) { cb->setCurrentIndex(i); break; }
+	cb->setEditText(index.data(Qt::EditRole).toString());
+}
+void ComboBoxDelegate::setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const{
+	if (index.column() != activeColumn) {
+		QItemDelegate::setModelData(editor, model, index);
+		return;
+	}
+	QComboBox* cb = qobject_cast<QComboBox*>(editor);
+	REQUIRE(cb);
+	model->setData(index, cb->currentText());
+}
+void ComboBoxDelegate::updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index) const{
+	if (index.column() != activeColumn) {
+		QItemDelegate::updateEditorGeometry(editor, option, index);
+		return;
+	}
+	editor->setGeometry(option.rect);
+}
+
+
+
 int ConfigDialog::lastUsedPage = 0;
 
-ConfigDialog::ConfigDialog(QWidget* parent): QDialog(parent), checkboxInternalPDFViewer(0), buildManager(0), riddled(false) {
+ConfigDialog::ConfigDialog(QWidget* parent): QDialog(parent), checkboxInternalPDFViewer(0), riddled(false), oldToolbarIndex(-1),mBuildManager(0) {
 	setModal(true);
 	ui.setupUi(this);
 
@@ -257,9 +329,14 @@ ConfigDialog::ConfigDialog(QWidget* parent): QDialog(parent), checkboxInternalPD
 	//ui.contentsWidget->setViewMode(QListView::ListMode);
 	//ui.contentsWidget->setMovement(QListView::Static);
 
+	//pageGeneral
+	connect(ui.pushButtonUpdateCheckNow, SIGNAL(clicked()), this, SLOT(updateCheckNow()));
+	connect(UpdateChecker::instance(), SIGNAL(checkCompleted()), this, SLOT(refreshLastUpdateTime()));
+	refreshLastUpdateTime();
+
 	//pageditor
-	QFontDatabase fdb;
-	ui.comboBoxFont->addItems(fdb.families());
+	populateComboBoxFont(false);
+	connect(ui.checkBoxShowOnlyMonospacedFonts, SIGNAL(toggled(bool)), this, SLOT(populateComboBoxFont(bool)));
 
 	ui.comboBoxEncoding->addItem("UTF-8");
 	foreach(int mib, QTextCodec::availableMibs()) {
@@ -275,11 +352,15 @@ ConfigDialog::ConfigDialog(QWidget* parent): QDialog(parent), checkboxInternalPD
 
 	connect(ui.btSelectThesaurusFileName, SIGNAL(clicked()), this, SLOT(browseThesaurus()));
 
+	connect(ui.pushButtonPathLog, SIGNAL(clicked()), this, SLOT(browsePathLog()));
+	connect(ui.pushButtonPathPdf, SIGNAL(clicked()), this, SLOT(browsePathPdf()));
+	connect(ui.pushButtonPathCommands, SIGNAL(clicked()), this, SLOT(browsePathCommands()));
+	
 	connect(ui.comboBoxThesaurusFileName, SIGNAL(editTextChanged(QString)), this, SLOT(comboBoxWithPathEdited(QString)));
 	connect(ui.comboBoxThesaurusFileName, SIGNAL(highlighted(QString)), this, SLOT(comboBoxWithPathHighlighted(QString)));
 
 
-	ui.labelGetDic->setText(tr("Get dictionaries at: %1").arg("<br><a href=\"http://wiki.services.openoffice.org/wiki/Dictionaries\">http://wiki.services.openoffice.org/wiki/Dictionaries</a>"));
+	ui.labelGetDic->setText(tr("Download Additional Dictionaries: %1").arg("<br><a href=\"http://wiki.services.openoffice.org/wiki/Dictionaries\">http://wiki.services.openoffice.org/wiki/Dictionaries</a>"));
 	ui.labelGetDic->setOpenExternalLinks(true);
 	//page custom environment
 	connect(ui.pbAddLine, SIGNAL(clicked()), this, SLOT(custEnvAddLine()));
@@ -288,42 +369,48 @@ ConfigDialog::ConfigDialog(QWidget* parent): QDialog(parent), checkboxInternalPD
 	connect(ui.pbRemoveSyntaxLine, SIGNAL(clicked()), this, SLOT(custSyntaxRemoveLine()));
 	environModes=0;
 	//pagequick
-	connect(ui.pushButtonQuickBuildWizard, SIGNAL(clicked()), SLOT(quickBuildWizard()));
-
-	connect(ui.pushButtonExecuteBeforeCompiling, SIGNAL(clicked()), this, SLOT(browsePrecompiling()));
-
+	connect(ui.pushButtonGrammarWordlists, SIGNAL(clicked()), this, SLOT(browseGrammarWordListsDir()));
+	connect(ui.pushButtonGrammarLTPath, SIGNAL(clicked()), this, SLOT(browseGrammarLTPath()));
+	connect(ui.pushButtonGrammarLTJava, SIGNAL(clicked()), this, SLOT(browseGrammarLTJavaPath()));
+	
 
 	fmConfig=new QFormatConfig(ui.formatConfigBox);
-	fmConfig->addCategory(tr("Basic highlighting")) << "normal" <<"background" <<"comment" <<"commentTodo" <<"keyword" <<"extra-keyword" <<"math-keyword"  <<"numbers" <<"text" <<"environment" <<"structure" <<"escapeseq"  << "verbatim" << "picture" << "sweave";
-	fmConfig->addCategory(tr("Inline checking")) <<"braceMatch" <<"braceMismatch" <<"styleHint" <<"spellingMistake" <<"latexSyntaxMistake" <<"referencePresent" <<"referenceMissing" <<"referenceMultiple" <<"citationPresent" <<"citationMissing"  <<"temporaryCodeCompletion";
+	fmConfig->addCategory(tr("Basic highlighting")) << "normal" <<"background" <<"comment" <<"commentTodo" <<"keyword" <<"extra-keyword" <<"math-delimiter" <<"math-keyword" <<"numbers" <<"text" <<"align-ampersand" <<"environment" <<"structure" <<"link" <<"escapeseq" << "verbatim" << "picture" << "sweave";
+	fmConfig->addCategory(tr("LaTeX checking")) <<"braceMatch" <<"braceMismatch" <<"latexSyntaxMistake" <<"referencePresent" <<"referenceMissing" <<"referenceMultiple" <<"citationPresent" <<"citationMissing"  <<"temporaryCodeCompletion";
+	fmConfig->addCategory(tr("Language checking")) <<"spellingMistake" << "wordRepetition" << "wordRepetitionLongRange"<< "badWord" << "grammarMistake" << "grammarMistakeSpecial1" << "grammarMistakeSpecial2" << "grammarMistakeSpecial3" << "grammarMistakeSpecial4";
 	fmConfig->addCategory(tr("Line highlighting"))     <<"line:error" <<"line:warning" <<"line:badbox" <<"line:bookmark" <<"line:bookmark0" <<"line:bookmark1" <<"line:bookmark2" <<"line:bookmark3" <<"line:bookmark4" <<"line:bookmark5" <<"line:bookmark6"  <<"line:bookmark7" <<"line:bookmark8"<<"line:bookmark9"<<"current";
 	fmConfig->addCategory(tr("Search")) <<"search"<<"replacement"<<"selection";
-        fmConfig->addCategory(tr("Diff")) <<"diffDelete"<<"diffAdd"<<"diffReplace";
+	fmConfig->addCategory(tr("Diff")) <<"diffDelete"<<"diffAdd"<<"diffReplace";
+	fmConfig->addCategory(tr("DTX files")) << "dtx:guard" << "dtx:macro" << "dtx:verbatim" << "dtx:specialchar";
+	fmConfig->addCategory(tr("Lua")) << "lua:keyword" << "lua:comment";
 
 	connect(ui.spinBoxSize, SIGNAL(valueChanged(int)), fmConfig, SLOT(setBasePointSize(int)));
 	
 	//fmConfig->setMaximumSize(490,300);
 	//fmConfig->setSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored);
-	(new QBoxLayout(QBoxLayout::TopToBottom, ui.formatConfigBox))->insertWidget(0,fmConfig);
-
+	QBoxLayout *layout = new QBoxLayout(QBoxLayout::TopToBottom, ui.formatConfigBox);
+	layout->setMargin(0);
+	layout->insertWidget(0,fmConfig);
 
 	ui.shortcutTree->setHeaderLabels(QStringList()<<tr("Command")<<tr("Default Shortcut")<<tr("Current Shortcut")<<tr("Additional Shortcut"));
 	ui.shortcutTree->setColumnWidth(0,200);
 	
 	//create icons
-    createIcon(tr("General"),QIcon(":/images/config_general.png"));
-    createIcon(tr("Commands"),QIcon(":/images/config_commands.png"));
-    createIcon(tr("Quick Build"),QIcon(":/images/config_quickbuild.png"));
-    createIcon(tr("Shortcuts"),QIcon(":/images/config_shortcut.png"));
-    createIcon(tr("Latex Menus"),QIcon(":/images/config_latexmenus.png"), true);
-    createIcon(tr("Toolbars"),QIcon(":/images/config_toolbars.png"), true);
-    createIcon(tr("Editor"),QIcon(":/images/config_editor.png"));
-    createIcon(tr("Adv. Editor"),QIcon(":/images/config_advancededitor.png"), true);
-    createIcon(tr("Custom Highlighting"),QIcon(":/images/config_highlighting.png"), true);
-    createIcon(tr("Completion"),QIcon(":/images/config_completion.png"));
-    createIcon(tr("Preview"),QIcon(":/images/config_preview.png"));
-    createIcon(tr("SVN"),QIcon(":/images/config_svn.png"));
-
+	createIcon(tr("General"),QIcon(":/images/config_general.png"));
+	createIcon(tr("Commands"),QIcon(":/images/config_commands.png"));
+	createIcon(tr("Build"),QIcon(":/images/config_quickbuild.png"));
+	createIcon(tr("Shortcuts"),QIcon(":/images/config_shortcut.png"));
+	createIcon(tr("Menus"),QIcon(":/images/config_latexmenus.png"), true);
+	createIcon(tr("Toolbars"),QIcon(":/images/config_toolbars.png"), true);
+	createIcon(tr("Editor"),QIcon(":/images/config_editor.png"));
+	createIcon(tr("Adv. Editor"),QIcon(":/images/config_advancededitor.png"), true);
+	createIcon(tr("Syntax Highlighting"),QIcon(":/images/config_highlighting.png"));
+	createIcon(tr("Custom Highlighting"),QIcon(":/images/config_highlighting.png"), true);
+	createIcon(tr("Completion"),QIcon(":/images/config_completion.png"));
+	createIcon(tr("Grammar"),QIcon(":/images/config_editor.png"));
+	createIcon(tr("Preview"),QIcon(":/images/config_preview.png"));
+	createIcon(tr("SVN"),QIcon(":/images/config_svn.png"));
+	
 	connect(ui.contentsWidget,
 	        SIGNAL(currentItemChanged(QListWidgetItem *, QListWidgetItem *)),
 	        this, SLOT(changePage(QListWidgetItem *, QListWidgetItem*)));
@@ -333,9 +420,11 @@ ConfigDialog::ConfigDialog(QWidget* parent): QDialog(parent), checkboxInternalPD
 
 	// custom toolbars
 	connect(ui.comboBoxToolbars,SIGNAL(currentIndexChanged(int)), SLOT(toolbarChanged(int)));
-	ui.listCustomToolBar->setIconSize(QSize(96, 96));
+	ui.listCustomToolBar->setIconSize(QSize(22, 22));
 	ui.listCustomToolBar->setViewMode(QListView::ListMode);
-	ui.listCustomToolBar->setMovement(QListView::Static);
+	ui.listCustomToolBar->setMovement(QListView::Snap);
+	ui.listCustomToolBar->setDragDropMode(QAbstractItemView::InternalMove);
+	connect(this,SIGNAL(accepted()), SLOT(checkToolbarMoved()));
 	connect(ui.pbToToolbar,SIGNAL(clicked()),this,SLOT(toToolbarClicked()));
 	connect(ui.pbFromToolbar,SIGNAL(clicked()),this,SLOT(fromToolbarClicked()));
 	ui.listCustomToolBar->setContextMenuPolicy(Qt::CustomContextMenu);
@@ -355,6 +444,8 @@ ConfigDialog::ConfigDialog(QWidget* parent): QDialog(parent), checkboxInternalPD
 	ui.checkBoxUseSystemTheme->setVisible(false);
 #endif
 	
+	ui.scrollAreaPaths->setVisible(false);
+	
 	QRect screen = QApplication::desktop()->screenGeometry();
 	if (!screen.isEmpty()) {
 		int nwidth = width(), nheight = height();
@@ -364,7 +455,7 @@ ConfigDialog::ConfigDialog(QWidget* parent): QDialog(parent), checkboxInternalPD
 		resize(nwidth, nheight);
 		move(frameGeometry().right() > screen.right()?screen.left():x(),
 		     frameGeometry().bottom() > screen.bottom()?screen.left():y());
-	}	
+	}
 }
 
 ConfigDialog::~ConfigDialog() {
@@ -393,27 +484,6 @@ void ConfigDialog::changePage(QListWidgetItem *current, QListWidgetItem *previou
 }
 
 
-//pageditor
-void ConfigDialog::browseDictDir() {
-	QString path=ui.leDictDir->text();
-	if (path.isEmpty()) path=QDir::homePath();
-	QString location = QFileDialog::getExistingDirectory(this, tr("Select dictionary directory"), path);
-	if (!location.isEmpty()) {
-		location.replace(QString("\\"),QString("/"));
-		ui.leDictDir->setText(location);
-	}
-}
-void ConfigDialog::dictDirChanged(const QString &newText) {
-	QString lang = ui.comboBoxSpellcheckLang->currentText();
-	ui.comboBoxSpellcheckLang->clear();
-	ui.comboBoxSpellcheckLang->addItems(SpellerManager::dictNamesForDir(newText));
-	// keep selected language if possible
-	int index = ui.comboBoxSpellcheckLang->findText(lang);
-	if (index >=0) {
-		ui.comboBoxSpellcheckLang->setCurrentIndex(index);
-	}
-}
-
 //sets the items of a combobox to the filenames and sub-directory names in the directory which name
 //is the current text of the combobox
 void ConfigDialog::comboBoxWithPathEdited(const QString& newText){
@@ -464,42 +534,116 @@ void ConfigDialog::comboBoxWithPathHighlighted(const QString& newText){
 }
 
 
-void ConfigDialog::browseThesaurus() {
-	QString path=ui.comboBoxThesaurusFileName->currentText();
-	if (path.isEmpty()) path=QDir::homePath();
-	QString location=QFileDialog::getOpenFileName(this,tr("Browse thesaurus database"),path,"Database (*.dat)",0,QFileDialog::DontResolveSymlinks);
-	if (!location.isEmpty()) {
-		location.replace(QString("\\"),QString("/"));
-		//	location="\""+location+"\"";
-		ui.comboBoxThesaurusFileName->setEditText(location);
-	}
-}
-
-void ConfigDialog::browsePrecompiling() {
-	QString path=ui.lineEditExecuteBeforeCompiling->text();
+bool ConfigDialog::browse(QWidget* w, const QString& title, const QString& extension, const QString& startPath, bool list){
+	QLineEdit* le = qobject_cast<QLineEdit*>(w);
+	QComboBox* cb = qobject_cast<QComboBox*>(w);
+	REQUIRE_RET(le || cb, false);
+	QString oldpath = le?le->text():cb->currentText();
+	QString path = oldpath;
+#ifdef Q_WS_WIN
+	QString pathSep = ";";
+#else
+	QString pathSep = ":";
+#endif
+	if (list && !path.isEmpty()) path = path.split(pathSep).last();
 	if (path.startsWith('"')) path.remove(0,1);
 	if (path.endsWith('"')) path.chop(1);
-	if (path.isEmpty()) path=QDir::rootPath();
-	QString location=QFileDialog::getOpenFileName(this,tr("Browse program"),path,"Program (*)",0,QFileDialog::DontResolveSymlinks);
-	if (!location.isEmpty()) {
-		location.replace(QString("\\"),QString("/"));
-		location="\""+location+"\"";
-		ui.lineEditExecuteBeforeCompiling->setText(location);
+	if (path.isEmpty()) path = startPath;
+	if (extension == "/") path = QFileDialog::getExistingDirectory(this, title, path);
+	else path = QFileDialog::getOpenFileName(this,title,path,extension,0,QFileDialog::DontResolveSymlinks);
+	if (!path.isEmpty()) {
+		if (!list) path.replace(QString("\\"),QString("/"));
+		else if (!oldpath.isEmpty()) path = oldpath + pathSep + path;
+		if (le) le->setText(path);
+		if (cb) cb->setEditText(path);
+		return true;
+	}
+	return false;
+}
+
+void ConfigDialog::browseThesaurus() {
+	browse(ui.comboBoxThesaurusFileName,tr("Browse thesaurus database"),"Database (*.dat)");
+}
+
+void ConfigDialog::browseGrammarWordListsDir(){
+	browse(ui.lineEditGrammarWordlists, tr("Select the grammar word lists dir"), "/");
+}
+void ConfigDialog::browseGrammarLTPath(){
+	browse(ui.lineEditGrammarLTPath, tr("Select the LanguageTool jar"), "Java-Program (*.jar)");
+}
+void ConfigDialog::browseGrammarLTJavaPath(){
+	browse(ui.lineEditGrammarLTJava, tr("Select java"), "Java (*)");
+}
+
+void ConfigDialog::browseDictDir() {
+	browse(ui.leDictDir, tr("Select dictionary directory"), "/");
+}
+void ConfigDialog::browsePathLog(){
+	browse(ui.lineEditPathLog, tr("Search Path for Logs"), "/", QDir::currentPath(), true);
+}
+void ConfigDialog::browsePathPdf(){
+	browse(ui.lineEditPathPDF, tr("Search Path for PDFs"), "/", QDir::currentPath(), true);
+}
+void ConfigDialog::browsePathCommands(){
+	browse(ui.lineEditPathCommands, tr("Search Path for Commands"), "/", QDir::rootPath(), true);
+}
+
+void ConfigDialog::dictDirChanged(const QString &newText) {
+	QString lang = ui.comboBoxSpellcheckLang->currentText();
+	ui.comboBoxSpellcheckLang->clear();
+	ui.comboBoxSpellcheckLang->addItems(SpellerManager::dictNamesForDir(newText));
+	// keep selected language if possible
+	int index = ui.comboBoxSpellcheckLang->findText(lang);
+	if (index >=0) {
+		ui.comboBoxSpellcheckLang->setCurrentIndex(index);
 	}
 }
 
-
-void ConfigDialog::quickBuildWizard(){
-	REQUIRE(buildManager);
-	ui.lineEditUserquick->setText(buildManager->editCommandList(ui.lineEditUserquick->text()));
-}
-
-void hideShowAdvancedOptions(QWidget* w, bool on){
+void ConfigDialog::hideShowAdvancedOptions(QWidget* w, bool on){
 	foreach (QObject* o, w->children()){
 		QWidget* w = qobject_cast<QWidget*> (o);
 		if (!w) continue;
 		if (w->property("advancedOption").isValid() && w->property("advancedOption").toBool())
 			w->setVisible(on);
+        // special treatment for metacommads
+        static QStringList simpleMetaOptions = QStringList() << "quick" << "compile" << "view" << "view-pdf" << "bibliography";
+        if(simpleMetaOptions.contains(w->objectName())){
+            QComboBox *cb=qobject_cast<QComboBox*> (o);
+            if(cb){
+                if(on){
+                    CommandMapping tempCommands=mBuildManager->getAllCommands();
+                    CommandInfo cmd=tempCommands.value(w->objectName());
+                    QString text=cb->currentText();
+                    int i=cb->findText(text);
+                    if(i>=0)
+                        text=cmd.metaSuggestionList.value(i,tr("<unknown>"));
+                    cb->clear();
+                    cb->addItems(cmd.metaSuggestionList);
+                    cb->setEditable(true);
+                    cb->setEditText(text);
+                }else{
+                    CommandMapping tempCommands=mBuildManager->getAllCommands();
+                    CommandInfo cmd=tempCommands.value(w->objectName());
+                    QString text=cb->currentText();
+                    int i=cmd.metaSuggestionList.indexOf(text);
+                    //if(i>=0)
+                     //   text=cmd.simpleDescriptionList.value(i,tr("<unknown>"));
+                    cb->clear();
+                    if(i>=0){
+                        foreach(QString elem,cmd.simpleDescriptionList){
+                            elem=qApp->translate("BuildManager",qPrintable(elem));
+                            cb->addItem(elem);
+                        }
+                        cb->setCurrentIndex(i);
+                        cb->setEditable(false);
+                    }else{
+                        cb->addItems(cmd.metaSuggestionList);
+                        cb->setEditable(true);
+                        cb->setEditText(text);
+                    }
+                }
+            }
+        }
 		hideShowAdvancedOptions(w,on);
 	}
 }
@@ -529,15 +673,20 @@ void ConfigDialog::advancedOptionsClicked(bool on){
 
 void ConfigDialog::toolbarChanged(int toolbar){
 	if (toolbar < 0 || toolbar >= customizableToolbars.size()) return;
+	checkToolbarMoved();
 	ui.listCustomToolBar->clear();
 	foreach (const QString& actName, customizableToolbars[toolbar]){
 		QAction* act=menuParent->findChild<QAction*>(actName);
 		QListWidgetItem *item;
-		if (act) item=new QListWidgetItem(act->icon(),act->text().replace("&",""));
-		else item=new QListWidgetItem(actName);
+		if (act) {
+			QIcon i = act->icon().pixmap(22,22);
+			item=new QListWidgetItem(i,act->text().replace("&",""));
+		} else item=new QListWidgetItem(actName);
 		item->setData(Qt::UserRole,actName);
+		if (actName == nameSeparator) item->setIcon(QIcon(":/images/separator.png"));
 		ui.listCustomToolBar->addItem(item);
 	}
+	oldToolbarIndex = toolbar;
 }
 
 void ConfigDialog::actionsChanged(int actionClass){
@@ -563,6 +712,7 @@ void ConfigDialog::actionsChanged(int actionClass){
 void ConfigDialog::toToolbarClicked(){
 	if (!ui.treePossibleToolbarActions->currentItem()) return;
 	if (ui.comboBoxToolbars->currentIndex() < 0 || ui.comboBoxToolbars->currentIndex() >= customizableToolbars.size()) return;
+	checkToolbarMoved();
 	QTreeWidgetItem *twi = ui.treePossibleToolbarActions->currentItem();
 	QString actName=twi->data(0,Qt::UserRole).toString();
 	QAction* act=menuParent->findChild<QAction*>(actName);
@@ -577,18 +727,23 @@ void ConfigDialog::toToolbarClicked(){
 void ConfigDialog::fromToolbarClicked(){
 	if(!ui.listCustomToolBar->currentItem()) return;
 	if (ui.comboBoxToolbars->currentIndex() < 0 || ui.comboBoxToolbars->currentIndex() >= customizableToolbars.size()) return;
-	/*QListWidgetItem *item=ui.listCustomToolBar->currentItem();
-    QList<QListWidgetItem *>result=ui.listCustomIcons->findItems(item->text(),Qt::MatchExactly);
-    foreach(QListWidgetItem *elem,result){
-        elem->setHidden(false);
-    }*/
+	checkToolbarMoved();
 	customizableToolbars[ui.comboBoxToolbars->currentIndex()].removeAt(ui.listCustomToolBar->currentRow());
 	delete ui.listCustomToolBar->takeItem(ui.listCustomToolBar->currentRow());
 }
 
+void ConfigDialog::checkToolbarMoved(){
+	if (oldToolbarIndex < 0 || oldToolbarIndex >= customizableToolbars.size()) return;
+	REQUIRE(ui.listCustomToolBar->count() == customizableToolbars[oldToolbarIndex].size());
+	for (int i=0;i<ui.listCustomToolBar->count();i++)	
+		customizableToolbars[oldToolbarIndex][i] = ui.listCustomToolBar->item(i)->data(Qt::UserRole).toString();
+		
+}
+
 void ConfigDialog::customContextMenuRequested(const QPoint &p){
 	QMenu menu;
-	menu.addAction(tr("Load other icon"),this, SLOT(loadOtherIcon()));
+	menu.addAction(tr("Load Other Icon"), this, SLOT(loadOtherIcon()));
+	menu.addAction(tr("Insert Separator"), this, SLOT(insertSeparator()));
 	menu.exec(ui.listCustomToolBar->mapToGlobal(p));
 }
 
@@ -607,6 +762,15 @@ void ConfigDialog::loadOtherIcon(){
 	}
 }
 
+void ConfigDialog::insertSeparator(){
+	int row = ui.listCustomToolBar->currentRow();
+	if (row<0) return;
+	ui.listCustomToolBar->insertItem(row, nameSeparator);
+	ui.listCustomToolBar->item(row)->setData(Qt::UserRole, nameSeparator);
+	ui.listCustomToolBar->item(row)->setIcon(QIcon(":/images/separator.png"));
+	customizableToolbars[ui.comboBoxToolbars->currentIndex()].insert(row, nameSeparator);
+}
+
 void ConfigDialog::populatePossibleActions(QTreeWidgetItem* parent, const QMenu* menu,bool keepHierarchy){
 	if (!menu) return;
 	QList<QAction *> acts=menu->actions();
@@ -618,22 +782,52 @@ void ConfigDialog::populatePossibleActions(QTreeWidgetItem* parent, const QMenu*
 		parent=twi;
 	}
 	for (int i=0; i<acts.size(); i++)
-		if (acts[i]->menu()) populatePossibleActions(parent, acts[i]->menu(),keepHierarchy);
-	else {
-		//if(acts[i]->data().isValid()){
-		QTreeWidgetItem* twi = new QTreeWidgetItem(parent,QStringList() << acts[i]->text().replace("&",""));
-		if (!acts[i]->isSeparator()){
-			if(!acts[i]->icon().isNull()) twi->setIcon(0,acts[i]->icon());
-			else twi->setIcon(0,QIcon(":/images/appicon.png"));
-		}
-		twi->setToolTip(0,acts[i]->text());
-		if (acts[i]->isSeparator()) twi->setData(0,Qt::UserRole,"separator");
-		else twi->setData(0,Qt::UserRole,acts[i]->objectName());
-		if (parent) parent->addChild(twi);
-		else ui.treePossibleToolbarActions->addTopLevelItem(twi);
+		if (acts[i]->menu()) {
+			populatePossibleActions(parent, acts[i]->menu(),keepHierarchy);
+		} else {
+			//if(acts[i]->data().isValid()){
+			QTreeWidgetItem* twi = new QTreeWidgetItem(parent,QStringList() << acts[i]->text().replace("&",""));
+			if (acts[i]->isSeparator()) {
+				twi->setIcon(0,QIcon(":/images/separator.png"));
+			} else {
+				if(!acts[i]->icon().isNull()) twi->setIcon(0,acts[i]->icon());
+				else twi->setIcon(0,QIcon(":/images/appicon.png"));
+			}
+			twi->setToolTip(0,acts[i]->text());
+			if (acts[i]->isSeparator()) twi->setData(0,Qt::UserRole, nameSeparator);
+			else twi->setData(0,Qt::UserRole,acts[i]->objectName());
+			if (parent) parent->addChild(twi);
+			else ui.treePossibleToolbarActions->addTopLevelItem(twi);
 	}
 }
 
+void ConfigDialog::updateCheckNow() {
+	UpdateChecker::instance()->check(false);
+}
+
+void ConfigDialog::refreshLastUpdateTime() {
+	ui.labelUpdateCheckDate->setText(UpdateChecker::lastCheckAsString());
+}
+
+void ConfigDialog::populateComboBoxFont(bool onlyMonospaced) {
+	QString currentFont = ui.comboBoxFont->currentText();
+	ui.comboBoxFont->clear();
+	QFontDatabase fdb;
+	if (onlyMonospaced) {
+		foreach(const QString &font, fdb.families()) {
+			if (fdb.isFixedPitch(font)) {
+				ui.comboBoxFont->addItem(font);
+			}
+		}
+	} else {
+		ui.comboBoxFont->addItems(fdb.families());
+	}
+	// restore font setting if possible
+	int idx = ui.comboBoxFont->findText(currentFont);
+	if (idx>=0) ui.comboBoxFont->setCurrentIndex(idx);
+
+}
+
 void ConfigDialog::custEnvAddLine(){
 	int i=ui.twHighlighEnvirons->rowCount();
 	ui.twHighlighEnvirons->setRowCount(i+1);
@@ -700,6 +894,12 @@ void ConfigDialog::custSyntaxRemoveLine(){
 }
 
 bool ConfigDialog::askRiddle(){
+	txsInformation(tr("You have enabled the advanced options. This allows to\n"
+				   "configure TXS in great detail. Disable the advanced\n"
+				   "options again to only view the most common settings."));
+	return true;
+
+	/*
 	QString solution = QInputDialog::getText(this, tr("Riddle"), tr(
 			"You come to a magic island where you meet three strange and wise friends. \n"
 			"One of them is always telling the truth, another one is always lying, and the third is deaf, so he answers randomly and cannot lie(!). \n"
@@ -715,6 +915,7 @@ bool ConfigDialog::askRiddle(){
 	if ((a1 && a2) || (a1 && a3) || (a2 && a3)) { QMessageBox::warning(this,tr("Riddle"),tr("Only one answer allowed")); return false; }
 	if (a3) return true;
 	return false;
+	*/
 }
 
 
