@@ -5,14 +5,9 @@
 //without having a relationship between completer and configmanager
 //so modifying one doesn't lead to a recompilation of the other
 
-struct Macro{
-	Macro();
-	Macro(const QString& nname, const QString& ntag, const QString& nabbrev, const QString& ntrigger);
-	QString name, tag, abbrev;
-	QString trigger;
-	QRegExp triggerRegex;
-	bool triggerLookBehind;
-};
+#include "modifiedQObject.h"
+
+#include "usermacro.h"
 
 
 class LatexCompleterConfig
@@ -30,7 +25,7 @@ public:
 	PreferedCompletionTab preferedCompletionTab;
 	QStringList words;
 	QMultiMap<uint,QPair<int,int> > usage;
-	
+
 	QList<Macro> userMacro;
 
 	void setFiles(const QStringList & newFiles);
