@@ -66,125 +66,6 @@ QDocumentSearch::Options QDocumentSearch::options() const
 
 
 //equal to next but needs matches
-bool QDocumentSearch::nextMatch(bool backward, bool again,  bool allowWrapAround){
-	Q_UNUSED(backward);Q_UNUSED(again);Q_UNUSED(allowWrapAround);
-#if 0
-	if (!hasOption(HighlightAll))
-		return false;
-	if (!indexedMatchCount()) searchMatches();
-	if (!indexedMatchCount()) {
-		m_cursor = QDocumentCursor();
-		return false;
-	}
-	if (m_cursor.isNull()){
-		m_cursor = QDocumentCursor();
-		return false; //need a starting point, m_cursor should have been set by next
-	}
-	//search the correct index	
-	if (m_index >= 0 && m_index < m_highlight.count() && m_highlight.at(m_index)->isWithinSelection(m_cursor)){
-		//we have a cached match!
-		if ( !backward ) ++m_index;
-		else --m_index;
-	} else if (backward){
-		//todo: binary search
-		m_index = matchLowerBoundUnequal(m_cursor.lineNumber(), m_cursor.columnNumber()+1);
-/*		m_index=-1;
-		for (int i=m_highlight.count()-1; i>=0;--i)
-			if (m_highlight[i]->selectionEnd()<=m_cursor) {
-				m_index=i;
-				break;
-			}*/
-	} else {
-		m_index = matchUpperBoundUnequal(m_cursor.lineNumber(), m_cursor.columnNumber()-1);
-		/*m_index=-1;
-		for (int i=0;i<m_highlight.count(); ++i)
-			if (m_highlight[i]->selectionStart()>=m_cursor) {
-				m_index=i;
-				break;
-			}*/
-	}
-
-	m_cursor = QDocumentCursor(); //don't need the old cursor anymore (and will set the new one later if found)
-	//remove invalid cursors
-
-	if ( m_index >= 0 && m_index < m_highlight.count() ) {
-		//TODO: understand why the selectedText().isEmpty() check is necessary, perhaps a bug in qdocumentcommand? (in the test case xxxxxxxxxxxxxxxx replace the cursor is moved to a not existing column)
-		if (backward) 
-			while (m_index>=0 && (m_highlight[m_index]->isNull() || m_highlight[m_index]->selectedText().isEmpty()))
-				delete m_highlight.takeAt(m_index--);
-		if (!backward)
-			while (m_index<m_highlight.count() && (m_highlight[m_index]->isNull() || m_highlight[m_index]->selectedText().isEmpty()))
-				delete m_highlight.takeAt(m_index);
-	}
-	
-	if ( (m_index < 0 || m_index >= m_highlight.count()) )
-	{
-		if (!allowWrapAround) 
-			return false;
-		
-		int ret = QMessageBox::Yes ;
-		if ( !hasOption(Silent) )
-		{
-			ret=QMessageBox::question(
-							m_editor,
-							tr("Failure"),
-							tr(
-								"End of matches reached.\n"
-								"Restart from the begining ?"
-							),
-							QMessageBox::Yes | QMessageBox::No,
-							QMessageBox::Yes
-						);
-		}
-		
-		if ( ret != QMessageBox::Yes ) 
-			return false;
-		
-		if (backward)  {
-			while (m_highlight.count() && (m_highlight.last()->isNull() || m_highlight.last()->selectedText().isEmpty()))
-				delete m_highlight.takeLast();
-			m_index=m_highlight.count()-1;
-		} else {
-			while (m_highlight.count() && (m_highlight.first()->isNull() || m_highlight.first()->selectedText().isEmpty()))
-				delete m_highlight.takeFirst();
-			m_index=0;
-		}
-		if (!m_highlight.count()) return false;
-	} 	
-
-	//create new cursor
-	//(don't copy cursor, because copying auto updated cursors is currently quite expensive)
-	if ( !backward )
-		m_cursor = m_editor->document()->cursor(m_highlight.at(m_index)->lineNumber(), m_highlight.at(m_index)->anchorColumnNumber(),
-							m_highlight.at(m_index)->lineNumber(), m_highlight.at(m_index)->columnNumber());
-	else
-		m_cursor = m_editor->document()->cursor(m_highlight.at(m_index)->lineNumber(), m_highlight.at(m_index)->columnNumber(),
-							m_highlight.at(m_index)->lineNumber(), m_highlight.at(m_index)->anchorColumnNumber());
-
-	if ( m_editor && !hasOption(Silent) ){
-		m_editor->setCursor(m_cursor);
-		if (m_cursor.line().isHidden()) m_editor->document()->expandParents(m_cursor.lineNumber());
-	}
-
-	if (hasOption(Replace) && !again) //again-replacement should already have been handled by next
-	{
-		QRegExp m_regexp=currentRegExp();
-		if (!m_regexp.exactMatch(m_cursor.selectedText())) 
-			return false; //wtf? anyway this check is necessary to access the submatch \1, \2 in the replace text
-		int ret=QMessageBox::Yes;
-		
-		if ( hasOption(Prompt) )
-			ret = QMessageBox::question(m_editor, tr("Replacement prompt"),
-								tr("Shall it be replaced?"),
-								QMessageBox::Yes | QMessageBox::No,
-								QMessageBox::Yes);
-
-		if (ret==QMessageBox::Yes)
-			replaceCursorText(m_regexp,backward);
-	}
-#endif
-	return true;
-}
 void QDocumentSearch::searchMatches(const QDocumentCursor& subHighlightScope, bool clearAll, bool clearSelection){
 	if ( !hasOption(HighlightAll) )
 		return;
@@ -195,8 +76,7 @@ void QDocumentSearch::searchMatches(const QDocumentCursor& subHighlightScope, bo
 	QDocument* d= currentDocument();
 	if ( !d || !d->lines()) return;
 
-	QFormatScheme *f = d->formatScheme() ? d->formatScheme() : QDocument::formatFactory();
-	int sid = f ? f->id("search") : 0;
+	int sid = d->getFormatId("search");
 	
 	if ( !sid ) {
 		qWarning("Highlighting of search matches disabled due to unavailability of a format scheme.");
@@ -271,7 +151,7 @@ void QDocumentSearch::searchMatches(const QDocumentCursor& subHighlightScope, bo
 	while ( !hc.atEnd() && hscope.isWithinSelection(hc))
 	{
 		int ln = hc.lineNumber();
-		//const QDocumentLine &l = hc.line();
+
 		QDocumentLine l = hc.line();
 		//TODO: Why is this needed ?????  It makes absolutely no sense, but if it isn't there, deleting a search text step by step and then adding the characters again, creates artifact overlays
 		l.setFlag(QDocumentLine::FormatsApplied, false);
@@ -301,14 +181,6 @@ void QDocumentSearch::searchMatches(const QDocumentCursor& subHighlightScope, bo
 	}
 
 	m_editor->viewport()->update();
-/*	if ( m_highlight.size() > oldHighlightCount)
-	{
-		//qDebug("%i matches in group %i", indexedMatchCount(), up);
-	} else if ( saved.empty() && m_highlight.empty() )
-		clearMatches();
-	if ( !saved.empty() )
-		m_highlight << saved;
-	m_index = -1; //current index became invalid due to (partially) cleaning (TODO?: change to new index)*/
 }
 
 /*!
@@ -323,8 +195,7 @@ void QDocumentSearch::clearMatches()
 	if ( !doc )
 		return;
 
-	QFormatScheme *f = doc->formatScheme() ? doc->formatScheme() : QDocument::formatFactory();
-	int sid = f ? f->id("search") : 0;
+	int sid = doc->getFormatId("search");
 
 	foreach (QDocumentLineHandle* h, m_highlights)
 		QDocumentLine(h).clearOverlays(sid);
@@ -332,19 +203,19 @@ void QDocumentSearch::clearMatches()
 	m_highlights.clear();
 	m_searchedScope = QDocumentCursor();
 	//qDebug("clearing matches");
-	//m_cursor = QDocumentCursor();
 
 	clearReplacements();
 	m_editor->viewport()->update();
 }
 
 void QDocumentSearch::clearReplacements(){
+	m_lastReplacedPosition = QDocumentCursor();
+	
 	QDocument* doc= currentDocument();
 	if ( !doc )
 		return;
 
-	QFormatScheme *f = doc->formatScheme() ? doc->formatScheme() : QDocument::formatFactory();
-	int sid = f ? f->id("replacement") : 0;
+	int sid = doc->getFormatId("replacement");
 
 	foreach (QDocumentLineHandle* l, m_highlightedReplacements)
 		QDocumentLine(l).clearOverlays(sid);
@@ -402,6 +273,9 @@ QString QDocumentSearch::searchText() const
 */
 void QDocumentSearch::setSearchText(const QString& f)
 {
+	if (m_string == f) 
+		return;
+	
 	m_string = f;
 	
 	if (hasOption(HighlightAll) && !m_string.isEmpty()) {
@@ -429,7 +303,8 @@ void QDocumentSearch::setOption(Option opt, bool on)
 	if ( ((m_option & opt) != 0) == on)
 		return; //no change, option already set
 
-	clearReplacements();
+	if (opt != QDocumentSearch::Replace) //don't clear replacement markers if toggling replace (important to jump back to last replacement if the panel is closed)
+		clearReplacements();
 
 	if ( on )
 		m_option |= opt;
@@ -488,8 +363,11 @@ QString QDocumentSearch::replaceText() const
 void QDocumentSearch::setReplaceText(const QString& r)
 {
 	m_replace = r;
-	//if (m_option & QDocumentSearch::HighlightAll)
-	//	clearMatches();
+	m_lastReplacedPosition = QDocumentCursor();
+}
+
+QDocumentCursor QDocumentSearch::lastReplacedPosition() const{
+	return m_lastReplacedPosition;
 }
 
 /*!
@@ -510,6 +388,7 @@ QDocumentCursor QDocumentSearch::cursor() const
 void QDocumentSearch::setCursor(const QDocumentCursor& c)
 {
 	m_cursor = c;
+	m_lastReplacedPosition = QDocumentCursor();
 }
 
 /*!
@@ -547,60 +426,6 @@ void QDocumentSearch::setScope(const QDocumentCursor& c)
 	highlightSelection();
 	if (hasOption(HighlightAll)) searchMatches();
 	else m_editor->viewport()->update();
-	//clearMatches();
-}
-
-/*!
-	\brief Test whether the end of the search scope has been reached
-*/
-bool QDocumentSearch::end(bool backward) const
-{
-	bool absEnd = backward ? m_cursor.atStart() : m_cursor.atEnd();
-	
-	if ( m_scope.isValid() && m_scope.hasSelection() )
-	{
-		absEnd |= !m_scope.isWithinSelection(m_cursor);
-		/*
-		qDebug(
-				"(%i, %i, %i) %s in {(%i, %i), (%i, %i)}",
-				m_cursor.lineNumber(),
-				m_cursor.anchorColumnNumber(),
-				m_cursor.columnNumber(),
-				absEnd ? "is not" : "is",
-				m_scope.selectionStart().lineNumber(),
-				m_scope.selectionStart().columnNumber(),
-				m_scope.selectionEnd().lineNumber(),
-				m_scope.selectionEnd().columnNumber()
-			);
-		*/
-	}
-	
-	return absEnd;
-}
-
-bool QDocumentSearch::end(bool backward,QDocumentLine l) const
-{
-	bool absEnd = backward ? l.lineNumber()==1 : l.lineNumber()==l.document()->lineCount();
-
-	if ( m_scope.isValid() && m_scope.hasSelection() )
-	{
-		absEnd |= !m_scope.isWithinSelection(m_cursor);
-		/*
-		qDebug(
-				"(%i, %i, %i) %s in {(%i, %i), (%i, %i)}",
-				m_cursor.lineNumber(),
-				m_cursor.anchorColumnNumber(),
-				m_cursor.columnNumber(),
-				absEnd ? "is not" : "is",
-				m_scope.selectionStart().lineNumber(),
-				m_scope.selectionStart().columnNumber(),
-				m_scope.selectionEnd().lineNumber(),
-				m_scope.selectionEnd().columnNumber()
-			);
-		*/
-	}
-
-	return absEnd;
 }
 
 /*!
@@ -612,19 +437,23 @@ bool QDocumentSearch::end(bool backward,QDocumentLine l) const
 
 	\note The search will start at the first character left/right from the selected text
 */
-int QDocumentSearch::next(bool backward, bool all, bool again, bool allowWrapAround)
+int QDocumentSearch::next(bool backward, bool all, bool again, bool allowWrapAround, const QDocumentCursor* overrideScope)
 {
 	if ( m_string.isEmpty() )
 		return 0;
+
+	if ( overrideScope && !overrideScope->isValid() ) overrideScope = 0;
+	
+	const QDocumentCursor& scope = overrideScope ? *overrideScope : m_scope;
 	
 	if ( m_cursor.isNull() )
 	{
-		if ( m_scope.isValid() && m_scope.hasSelection() )
+		if ( scope.isValid() && scope.hasSelection() )
 		{
 			if ( backward )
-				m_cursor = m_scope.selectionEnd();
+				m_cursor = scope.selectionEnd();
 			else
-				m_cursor = m_scope.selectionStart();
+				m_cursor = scope.selectionStart();
 		} else if ( m_editor ) {
 			
 			m_cursor = QDocumentCursor(m_editor->document());
@@ -639,11 +468,6 @@ int QDocumentSearch::next(bool backward, bool all, bool again, bool allowWrapAro
 
 	QDocumentCursor firstMatch;
 
-	/*if (all && !hasOption(Replace)) {
-		all=false;
-		qWarning("QDocumentSearch:next: all without replace is meaningless");
-	}*/
-
 	QRegExp m_regexp=currentRegExp();
 	
 	int replaceCount = 0;
@@ -651,6 +475,7 @@ int QDocumentSearch::next(bool backward, bool all, bool again, bool allowWrapAro
 	if (hasOption(Replace) && again && !all) 
 		if (m_regexp.exactMatch(m_cursor.selectedText()))  {
 			replaceCursorText(m_regexp,backward);
+			updateReplacementOverlays();
 			replaceCount++;
 			//foundCount++; we can't set this here, because it has to search the next match
 			//and if (foundCount) is true, it thinks already found something and doesn't restart from scope
@@ -658,11 +483,11 @@ int QDocumentSearch::next(bool backward, bool all, bool again, bool allowWrapAro
 
 	//ensure that the current selection isn't searched
 	if ( m_cursor.hasSelection() ) {
-		if (m_cursor.selectionStart() == m_scope.selectionStart() &&
-			m_cursor.selectionEnd() == m_scope.selectionEnd()) {
+		if (m_cursor.selectionStart() == scope.selectionStart() &&
+			m_cursor.selectionEnd() == scope.selectionEnd()) {
 			//search whole scope
-			if (backward) m_cursor=m_scope.selectionEnd();
-			else m_cursor=m_scope.selectionStart();
+			if (backward) m_cursor=scope.selectionEnd();
+			else m_cursor=scope.selectionStart();
 		} else {
 			if (backward ^ all) //let all search in the selection
 				m_cursor=m_cursor.selectionStart();
@@ -671,46 +496,45 @@ int QDocumentSearch::next(bool backward, bool all, bool again, bool allowWrapAro
 		}
 	}
 	
-	QDocumentSelection boundaries;
-	bool bounded = m_scope.isValid() && m_scope.hasSelection();
+	QDocumentSelection boundaries = {0,0,0,0};
+	bool bounded = scope.isValid() && scope.hasSelection();
 	
 	// condition only to avoid debug messages...
 	if ( bounded ) {
-		boundaries = m_scope.selection();
+		boundaries = scope.selection();
 	
 		//moves the cursor in the search scope if it isn't there, but directly in front of the selection (only possible if there actually is a selection)
-		if ( end(backward) ) {
-			if ( !backward && m_cursor < m_scope.selectionStart() ) {
-				m_cursor = m_scope.selectionStart();
+		if (   ( backward ? m_cursor.atStart() : m_cursor.atEnd() )                              //absEnd
+		    || ( scope.isValid() && scope.hasSelection() && !scope.isWithinSelection(m_cursor) )
+		   ) {
+			if ( !backward && m_cursor < scope.selectionStart() ) {
+				m_cursor = scope.selectionStart();
 			} else {
-				if ( backward && m_cursor > m_scope.selectionEnd() )
-					m_cursor = m_scope.selectionEnd();
+				if ( backward && m_cursor > scope.selectionEnd() )
+					m_cursor = scope.selectionEnd();
 			}
 		}
 	}
-
-	//if (hasOption(HighlightAll) && !all)  //special handling if highlighting is on, but all replace is still handled here
-	//	return nextMatch(backward,again,allowWrapAround);
-	
-	/*qDebug(
-		"searching %s from line %i (column %i)",
-		backward ? "backward" : "forward",
-		m_cursor.lineNumber(),
-		m_cursor.columnNumber()
-	);
-	//*/
 	
 	bool realReplace=hasOption(Replace) && (!again || all);
 	
 	QDocumentCursor::MoveOperation move;
-//	QDocument *d = currentDocument();
 	
 	move = backward ? QDocumentCursor::PreviousBlock : QDocumentCursor::NextBlock;	
 	
 	int foundCount = 0;
 
+    if (m_editor && all && !hasOption(Prompt)){
+        m_editor->document()->beginMacro();
+		m_editor->document()->beginDelayedUpdateBlock();
+    }
+	
 	m_cursor.setColumnMemory(false);
-	while ( !end(backward) )
+	QDocumentCursor lastSelection;
+	while ( !( 
+	             ( backward ? m_cursor.atStart() : m_cursor.atEnd() )                              //absEnd
+	          || ( scope.isValid() && scope.hasSelection() && !scope.isWithinSelection(m_cursor) )
+	      ) )
 	{
 		if ( backward && !m_cursor.columnNumber() )
 		{
@@ -768,8 +592,10 @@ int QDocumentSearch::next(bool backward, bool all, bool again, bool allowWrapAro
 				}
 				
 				if ( m_editor && !hasOption(Silent)) {
-					m_editor->setCursor(m_cursor);
-					m_editor->ensureCursorVisibleSurrounding();
+					if (!all || hasOption(Prompt)) {
+						m_editor->setCursor(m_cursor);
+						m_editor->ensureCursorVisibleSurrounding();
+					} else lastSelection = m_cursor;
 				}
 				
 				foundCount++;
@@ -793,13 +619,13 @@ int QDocumentSearch::next(bool backward, bool all, bool again, bool allowWrapAro
 						rep=ret==QMessageBox::Yes;
 						if (ret==QMessageBox::Cancel) {
 							QMessageBox::information(m_editor,tr("Replacing Canceled"),tr("%1 (of %2 found so far) occurences have been replaced").arg(replaceCount).arg(foundCount),QMessageBox::Ok);
-							//rep=all=false;//return false;
 							return foundCount;
 						}
 					}
 					
 					if ( rep ) {
 						replaceCursorText(m_regexp,backward);
+						if (!all) updateReplacementOverlays();
 						replaceCount++;
 					}
 				} 
@@ -810,15 +636,18 @@ int QDocumentSearch::next(bool backward, bool all, bool again, bool allowWrapAro
 			}
 		} else 
 			m_cursor.movePosition(1, move, QDocumentCursor::ThroughFolding);
-			/*if(backward) {
-				l=m_editor->document()->line(ln-1);
-				m_cursor.setLineNumber(ln-1);
-				if(l.isValid()) m_cursor.setColumnNumber(l.length());
-			} else {
-				l=m_editor->document()->line(ln+1);
-				m_cursor.setLineNumber(ln+1);
-				m_cursor.setColumnNumber(0);
-			}*/
+	}
+	if ( all && replaceCount )
+		updateReplacementOverlays();
+	if ( m_editor && all ) {
+		if (!hasOption(Prompt)) {
+			m_editor->document()->endDelayedUpdateBlock();
+            m_editor->document()->endMacro();
+			if (!hasOption(Silent)){
+				m_editor->setCursor(lastSelection);
+				m_editor->ensureCursorVisibleSurrounding();
+			}
+		}
 	}
 		
 	if ( !foundCount && allowWrapAround)
@@ -861,20 +690,16 @@ int QDocumentSearch::next(bool backward, bool all, bool again, bool allowWrapAro
 							);
 			if ( ret == QMessageBox::Yes )
 			{
-				QDocumentCursor oldScope(m_scope, true);
 				QDocumentCursor newScope(backward?firstMatch.selectionEnd():firstMatch.selectionStart());
 				if ( !backward ) newScope.movePosition(0,QDocumentCursor::Start,QDocumentCursor::KeepAnchor);
 				else newScope.movePosition(0,QDocumentCursor::End,QDocumentCursor::KeepAnchor);
-				if (m_scope.isValid() && m_scope.hasSelection()) m_scope = m_scope.intersect(newScope);
-				else m_scope = newScope;
+				if (scope.isValid() && scope.hasSelection()) newScope = scope.intersect(newScope);			
 				m_cursor = QDocumentCursor();
 
 				int result = foundCount;
-				if (m_scope.isValid() && m_scope.hasSelection() )
-					result += next(backward, all, again, false);
+				if (newScope.isValid() && newScope.hasSelection() )
+					result += next(backward, all, again, false, &newScope);
 
-				m_scope = oldScope;
-				m_scope.setAutoUpdated(true);
 				return result;
 			}
 		} else if ( !hasOption(Silent) )
@@ -904,8 +729,9 @@ static QString escapeCpp(const QString& s)
 				es += '\n';
 			else if ( c == 'r' )
 				es += '\r';
-			else if ( c == '0' )
-				es += '\0';
+			//else if ( c == '0' )
+			//	es += '\0';
+			else es += '\\', es += c;
 
 		} else {
 			es += s.at(i);
@@ -925,53 +751,34 @@ void QDocumentSearch::replaceCursorText(QRegExp& m_regexp,bool backward){
 			replacement.replace(QString("\\") + QString::number(i),
 								m_regexp.cap(i));
 
-	//highlight replacement, save old overlays
-	//old overlays
-	QDocument* d = m_cursor.document();
-	int rid = 0;
-	int shift = 0;
-	QList<QFormatRange> oldOverlaysBefore, oldOverlaysAfter;
-	if (hasOption(HighlightReplacements)) {
-		QFormatScheme *f = d ? d->formatScheme() : QDocument::formatFactory();
-		rid = f ? f->id("replacement") : 0;
-		if (rid) {
-			QDocumentSelection boundaries = m_cursor.selection();
-			shift = -boundaries.end;
-			QDocumentLine startLine = d->line(boundaries.startLine);
-			QDocumentLine endLine = d->line(boundaries.endLine);
-			oldOverlaysBefore = startLine.getOverlays(rid);
-			for (int i=oldOverlaysBefore.size()-1;i>=0;i--)
-				if (oldOverlaysBefore[i].offset + oldOverlaysBefore[i].length > boundaries.start)
-					oldOverlaysBefore.removeAt(i);
-			oldOverlaysAfter = endLine.getOverlays(rid);
-			for (int i=oldOverlaysAfter.size()-1;i>=0;i--)
-				if (oldOverlaysAfter[i].offset < boundaries.end)
-					oldOverlaysAfter.removeAt(i);
-			startLine.clearOverlays(rid);
-			endLine.clearOverlays(rid);
-		}
-	}
-	/*m_replaced=2;
-	int n=replacement.lastIndexOf("\n");
-	QString replacement_lastLine=replacement.mid(n+1);
-	m_replaceDeltaLength=replacement_lastLine.length()-m_cursor.selectedText().length();
-	m_replaceDeltaLines=replacement.count("\n");*/
+	QDocumentSelection old_boundaries = m_cursor.selection();
 	m_cursor.replaceSelectedText(replacement);
+	m_newReplacementOverlays << QPair<QDocumentSelection,QDocumentSelection>(old_boundaries, m_cursor.selection());
 
-	//highlight replacement
-	if (rid) {
-		//restore old
-		QDocumentSelection boundaries = m_cursor.selection();
-		shift += boundaries.end;
+	m_lastReplacedPosition = m_cursor;
+	
+	//make sure that the cursor if  the correct side of the selection is used
+	//(otherwise the cursor could be moved out of the searched scope by a long 
+	//replacement text)
+	if (m_cursor.hasSelection()) {
+		if (backward) m_cursor=m_cursor.selectionStart();
+		else m_cursor=m_cursor.selectionEnd();
+	}
+}
 
+void QDocumentSearch::updateReplacementOverlays(){
+	if (m_newReplacementOverlays.isEmpty()) return;
+	QDocument* d = m_cursor.document();
+	if (!d) return;
+	int rid = d->getFormatId("replacement");
+	if (!d || !hasOption(HighlightReplacements) || !rid)  { 
+		m_newReplacementOverlays.clear();
+		return;
+	}
+	for (int i=0;i<m_newReplacementOverlays.size();i++) {
+		const QDocumentSelection& boundaries = m_newReplacementOverlays[i].second;
 		QDocumentLine startLine = d->line(boundaries.startLine);
 		QDocumentLine endLine = d->line(boundaries.endLine);
-		foreach (const QFormatRange& frb, oldOverlaysBefore)
-			startLine.addOverlay(frb);
-		foreach (const QFormatRange& fra, oldOverlaysAfter)
-			endLine.addOverlay(QFormatRange(fra.offset+shift,fra.length,fra.format));
-
-		//add new overlay
 		m_highlightedReplacements.insert(startLine.handle());
 		if (boundaries.startLine == boundaries.endLine)  //single line replacement
 			startLine.addOverlay(QFormatRange(boundaries.start, boundaries.end - boundaries.start, rid));
@@ -987,14 +794,49 @@ void QDocumentSearch::replaceCursorText(QRegExp& m_regexp,bool backward){
 			}
 		}
 	}
-
-	//make sure that the cursor if  the correct side of the selection is used
-	//(otherwise the cursor could be moved out of the searched scope by a long 
-	//replacement text)
-	if (m_cursor.hasSelection()) {
-		if (backward) m_cursor=m_cursor.selectionStart();
-		else m_cursor=m_cursor.selectionEnd();
+	/*
+	//highlight replacement, save old overlays
+	//old overlays
+	QList<QFormatRange> oldOverlays;
+	int firstLine = m_newReplacementOverlays.first().first.startLine;
+	int endLine = qMin(m_newReplacementOverlays.last().first.endLine, d->lineCount()-1);
+	for (int l = firstLine; l<=endLine; l++) {
+		
 	}
+
+	QDocumentSelection boundaries = m_cursor.selection();
+			shift = -boundaries.end;
+	
+	QDocumentLine startLine = d->line(boundaries.startLine);
+	QDocumentLine endLine = d->line(boundaries.endLine);
+			oldOverlaysBefore = startLine.getOverlays(rid);
+			for (int i=oldOverlaysBefore.size()-1;i>=0;i--)
+				if (oldOverlaysBefore[i].offset + oldOverlaysBefore[i].length > boundaries.start)
+					oldOverlaysBefore.removeAt(i);
+			oldOverlaysAfter = endLine.getOverlays(rid);
+			for (int i=oldOverlaysAfter.size()-1;i>=0;i--)
+				if (oldOverlaysAfter[i].offset < boundaries.end)
+					oldOverlaysAfter.removeAt(i);
+			startLine.clearOverlays(rid);
+			endLine.clearOverlays(rid);
+		}
+	}
+
+	//highlight replacement
+	if (rid) {
+		//restore old
+		QDocumentSelection boundaries = m_cursor.selection();
+		shift += boundaries.end;
+
+		QDocumentLine startLine = d->line(boundaries.startLine);
+		QDocumentLine endLine = d->line(boundaries.endLine);
+		foreach (const QFormatRange& frb, oldOverlaysBefore)
+			startLine.addOverlay(frb);
+		foreach (const QFormatRange& fra, oldOverlaysAfter)
+			endLine.addOverlay(QFormatRange(fra.offset+shift,fra.length,fra.format));
+
+		//add new overlay
+	}*/
 }
 
 void QDocumentSearch::documentContentChanged(int line, int n){
@@ -1021,7 +863,6 @@ void QDocumentSearch::documentContentChanged(int line, int n){
 	c.setColumnNumber(le.length(), QDocumentCursor::KeepAnchor);
 	highlightSelection(c);
 	searchMatches(c,false);
-	//searchMatches();
 }
 
 void QDocumentSearch::visibleLinesChanged(){
@@ -1034,8 +875,9 @@ void QDocumentSearch::highlightSelection(const QDocumentCursor& subHighlightScop
 	if (hasOption(Silent))
 		return;
 
-	QFormatScheme *f = m_editor->document()->formatScheme() ? m_editor->document()->formatScheme() : QDocument::formatFactory();
-	int sid = f ? f->id("selection") : 0;
+	if (!m_editor || !m_editor->document()) return;
+
+	int sid = m_editor->document()->getFormatId("selection");
 	if (!sid) return;
 
 	int begLine, endLine, begCol, endCol;
@@ -1061,7 +903,6 @@ void QDocumentSearch::highlightSelection(const QDocumentCursor& subHighlightScop
 		m_highlightedScope = m_scope;
 		m_highlightedScope.setAutoUpdated(true);
 	}
-	//m_editor->viewport()->update();
 }
 
 void QDocumentSearch::lineDeleted(QDocumentLineHandle* line){
