@@ -1,4 +1,4 @@
-	/****************************************************************************
+/****************************************************************************
 **
 ** Copyright (C) 2006-2009 fullmetalcoder <fullmetalcoder@hotmail.fr>
 **
@@ -213,8 +213,9 @@ class QCE_EXPORT QEditor : public QAbstractScrollArea
 		
 		QList<QEditorInputBindingInterface*> inputBindings() const;
 		
-		bool isCursorVisible() const;
+		Q_INVOKABLE bool isCursorVisible() const;
 		QDocumentCursor cursor() const;
+		Q_PROPERTY(QDocumentCursor cursor READ cursor WRITE setCursor DESIGNABLE false STORED false);
 		QDocumentCursorHandle* cursorHandle() const;
 		
 		Q_INVOKABLE int cursorMirrorCount() const;
@@ -230,6 +231,8 @@ class QCE_EXPORT QEditor : public QAbstractScrollArea
 		virtual QRect lineRect(int line) const;
 		virtual QRect lineRect(const QDocumentLine& l) const;
 		virtual QRect cursorRect(const QDocumentCursor& c) const;
+		virtual QRect cursorMircoFocusRect() const;
+
 		
 		virtual int getFirstVisibleLine();
 		virtual int getLastVisibleLine();
@@ -237,14 +240,12 @@ class QCE_EXPORT QEditor : public QAbstractScrollArea
 		Q_INVOKABLE virtual void scrollToFirstLine(int l);
 
 		void setCursorSurroundingLines(int s);
-		
-		#ifndef _QMDI_
+			
 		Q_INVOKABLE QString name() const;
 		Q_INVOKABLE QString fileName() const;
 		Q_INVOKABLE QFileInfo fileInfo() const;
 		
 		Q_INVOKABLE bool isContentModified() const;
-		#endif
 		
 		Q_INVOKABLE bool isInConflict() const;
 		Q_INVOKABLE QTextCodec* getFileCodec() const;
@@ -349,11 +350,12 @@ class QCE_EXPORT QEditor : public QAbstractScrollArea
 		void setHardLineWrapping(bool on);
 		void setSoftLimitedLineWrapping(bool on);
 		void setWrapLineWidth(int l);
-		
+public:		
 		virtual void save();
 		void save(const QString& filename);
 		bool saveCopy(const QString& filename);
-
+		void saveEmergencyBackup(const QString& filename);
+public slots:
 		
 		virtual void print();
 		
@@ -366,7 +368,7 @@ class QCE_EXPORT QEditor : public QAbstractScrollArea
 		
 		void load(const QString& file, QTextCodec* codec/*= QTextCodec::codecForLocale()*/);
 		
-		void setText(const QString& s);
+		void setText(const QString& s, bool allowUndo = true);
 		
 		void setDocument(QDocument *d);
 		
@@ -435,6 +437,9 @@ class QCE_EXPORT QEditor : public QAbstractScrollArea
 		void emitWordDoubleClicked();
 		
 		void visibleLinesChanged();
+		
+		void slowOperationStarted();
+		void slowOperationEnded();
 	public slots:
 		void checkClipboard();
 		void reconnectWatcher();
@@ -457,6 +462,7 @@ class QCE_EXPORT QEditor : public QAbstractScrollArea
 		virtual void timerEvent(QTimerEvent *e);
 		
 		virtual void keyPressEvent(QKeyEvent *e);
+		virtual void keyReleaseEvent(QKeyEvent *);
 	public:
 		virtual void inputMethodEvent(QInputMethodEvent* e);
 		
@@ -490,6 +496,8 @@ class QCE_EXPORT QEditor : public QAbstractScrollArea
 		virtual QMimeData* createMimeDataFromSelection() const;
 		
 		virtual void scrollContentsBy(int dx, int dy);
+
+		virtual QVariant inputMethodQuery(Qt::InputMethodQuery property) const;
 	public:
 		virtual void insertFromMimeData(const QMimeData *d);
 
@@ -504,7 +512,7 @@ class QCE_EXPORT QEditor : public QAbstractScrollArea
 		void ensureVisible(const QRect &rect);
 		
 	protected:
-		void preInsert(QDocumentCursor& c, const QString& text);
+		void preInsertUnindent(QDocumentCursor& c, const QString& text, int additionalUnindent);
 
 	public slots:
 		void insertText(QDocumentCursor& c, const QString& text);
@@ -525,6 +533,7 @@ class QCE_EXPORT QEditor : public QAbstractScrollArea
 		void clearCursorMirrors();
 		void addCursorMirror(const QDocumentCursor& c);
 
+		void setCursorBold(bool bold);
 
 		QString cutBuffer;
 
@@ -553,12 +562,7 @@ class QCE_EXPORT QEditor : public QAbstractScrollArea
 		
 		void init(bool actions = true,QDocument *doc=0);
 		void updateBindingsMenu();
-		
-		#ifndef _QMDI_
-		QString m_name, m_fileName;
-		QFileInfo m_fileInfo; 
-		#endif
-		
+			
 		QMenu *pMenu;
 		QHash<QString, QAction*> m_actions;
 		
@@ -582,6 +586,8 @@ class QCE_EXPORT QEditor : public QAbstractScrollArea
 		
 		QList<QDocumentCursor> m_mirrors;
 		
+		bool atPlaceholder();
+        bool isMirrored();
 		int m_curPlaceHolder, m_lastPlaceHolder, m_cphOffset;
 		bool m_placeHolderSynchronizing; 
 		
@@ -623,4 +629,8 @@ class QCE_EXPORT QEditor : public QAbstractScrollArea
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(QEditor::State);
 
+inline bool QEditor::atPlaceholder() {
+	return m_curPlaceHolder >= 0 && m_curPlaceHolder<m_placeHolders.count();
+}
+
 #endif
