@@ -1,3 +1,11 @@
+var metaData = {
+"Name"        : "Longtable",
+"Description" : "Format table using the longtable environonment.", 
+"Author"      : "Jan Sundermeyer",
+"Date"        : "4.9.2011",
+"Version"     : "1.0"
+}
+
 function print(str){
 cursor.insertText(str)
 }
