@@ -10,6 +10,49 @@
 #include <QtTest/QtTest>
 TableManipulationTest::TableManipulationTest(QEditor* editor): ed(editor){}
 
+void TableManipulationTest::splitColDef_data(){
+	QTest::addColumn<QString>("def");
+	QTest::addColumn<QString>("splittedDef"); // for simplicity use a single string instead of a QStringList and '-' as separator
+
+	QTest::newRow("one col") << "l" << "l";
+	QTest::newRow("few cols") << "lrrc" << "l-r-r-c";
+	QTest::newRow("hrows") << "l|rr|c" << "l-|r-r-|c";
+	QTest::newRow("allHrows") << "|l|r|" << "|l-|r|";
+	QTest::newRow("supportedChars") << "lcrsSp" << "l-c-r-s-S-p";
+	QTest::newRow("atOperator") << "l@{someText}r" << "l-@{someText}r";
+	QTest::newRow("repetion") << "*{3}{lr}" << "l-r-l-r-l-r";
+	QTest::newRow("block") << "p{width}l" << "p{width}-l";
+	QTest::newRow("option") << "rs[key=val]l" << "r-s[key=val]-l";
+}
+
+void TableManipulationTest::splitColDef(){
+	QFETCH(QString, def);
+	QFETCH(QString, splittedDef);
+
+	QEQUAL(LatexTables::splitColDef(def).join("-"), splittedDef);
+}
+
+void TableManipulationTest::simplifyColDef_data(){
+	// for simplicity use a single string instead of a QStringList and '-' as separator
+	QTest::addColumn<QString>("def");
+	QTest::addColumn<QString>("simplified");
+
+	QTest::newRow("hline") << "|l-|r-r-|c|" << "l-r-r-c";
+	QTest::newRow("atOperator") << "l-@{someText}r" << "l-r";
+	QTest::newRow("block") << "p{width}-l" << "p-l";
+	QTest::newRow("option") << "r-s[key=val]-l" << "r-s-l";
+}
+
+void TableManipulationTest::simplifyColDef(){
+	QFETCH(QString, def);
+	QFETCH(QString, simplified);
+
+	QStringList res = def.split('-');
+	LatexTables::simplifyColDefs(res);
+
+	QEQUAL(res.join("-"), simplified);
+}
+
 void TableManipulationTest::addCol_data(){
 	QTest::addColumn<QString>("text");
 	QTest::addColumn<int>("row");
@@ -45,7 +88,7 @@ void TableManipulationTest::addCol(){
 	QFETCH(int, col);
 	QFETCH(QString, newText);
 	
-	ed->setText(text);
+	ed->setText(text, false);
 	LatexTables::addColumn(ed->document(),row,col,0);
 
 	QEQUAL(ed->document()->text(), newText);
@@ -86,7 +129,7 @@ void TableManipulationTest::addRow(){
 	QFETCH(int, col);
 	QFETCH(QString, newText);
 
-	ed->setText(text);
+	ed->setText(text, false);
 	ed->setCursorPosition(row,col);
 	QDocumentCursor c(ed->cursor());
 	LatexTables::addRow(c,2);
@@ -113,24 +156,24 @@ void TableManipulationTest::remCol_data(){
 		<< "\\begin{tabular}{x}\na\\\\\nc\\\\\ne\\\\\n\\end{tabular}\n";
 
 	QTest::newRow("rem col 0, multicolumn")
-		<< "\\begin{tabular}{ll}\na&b\\\\\n\\multicolumn{2}{c}\\\\\ne&f\\\\\n\\end{tabular}\n"
+		<< "\\begin{tabular}{ll}\na&b\\\\\n\\multicolumn{2}{c}{txt}\\\\\ne&f\\\\\n\\end{tabular}\n"
 		<< 1 << 0
-		<< "\\begin{tabular}{l}\nb\\\\\n\\multicolumn{1}{c}\\\\\nf\\\\\n\\end{tabular}\n";
+		<< "\\begin{tabular}{l}\nb\\\\\n\\multicolumn{1}{c}{txt}\\\\\nf\\\\\n\\end{tabular}\n";
 
 	QTest::newRow("rem col 1, multicolumn")
-		<< "\\begin{tabular}{ll}\na&b\\\\\n\\multicolumn{2}{c}\\\\\ne&f\\\\\n\\end{tabular}\n"
+		<< "\\begin{tabular}{ll}\na&b\\\\\n\\multicolumn{2}{c}{txt}\\\\\ne&f\\\\\n\\end{tabular}\n"
 		<< 1 << 1
-		<< "\\begin{tabular}{l}\na\\\\\n\\multicolumn{1}{c}\\\\\ne\\\\\n\\end{tabular}\n";
+		<< "\\begin{tabular}{l}\na\\\\\n\\multicolumn{1}{c}{txt}\\\\\ne\\\\\n\\end{tabular}\n";
 
 	QTest::newRow("rem col 1, multicolumn plus col")
-		<< "\\begin{tabular}{ll}\na&b\\\\\n\\multicolumn{2}{c}&d\\\\\ne&f\\\\\n\\end{tabular}\n"
+		<< "\\begin{tabular}{ll}\na&b\\\\\n\\multicolumn{2}{c}{txt}&d\\\\\ne&f\\\\\n\\end{tabular}\n"
 		<< 1 << 1
-		<< "\\begin{tabular}{l}\na\\\\\n\\multicolumn{1}{c}&d\\\\\ne\\\\\n\\end{tabular}\n";
+		<< "\\begin{tabular}{l}\na\\\\\n\\multicolumn{1}{c}{txt}&d\\\\\ne\\\\\n\\end{tabular}\n";
 
 	QTest::newRow("rem col 2, multicolumn")
-		<< "\\begin{tabular}{ll}\na&b\\\\\n\\multicolumn{2}{c}&d\\\\\ne&f\\\\\n\\end{tabular}\n"
+		<< "\\begin{tabular}{ll}\na&b\\\\\n\\multicolumn{2}{c}{txt}&d\\\\\ne&f\\\\\n\\end{tabular}\n"
 		<< 1 << 2
-		<< "\\begin{tabular}{ll}\na&b\\\\\n\\multicolumn{2}{c}\\\\\ne&f\\\\\n\\end{tabular}\n";
+		<< "\\begin{tabular}{ll}\na&b\\\\\n\\multicolumn{2}{c}{txt}\\\\\ne&f\\\\\n\\end{tabular}\n";
 
 	QTest::newRow("rem col 0, row over multiple lines")
 		<< "\\begin{tabular}{ll}\na&\nb\\\\\nc\n&\nd\\\\\ne&f\\\\\n\\end{tabular}\n"
@@ -169,11 +212,11 @@ void TableManipulationTest::remCol(){
 	QFETCH(int, col);
 	QFETCH(QString, newText);
 
-	ed->setText(text);
+	ed->setText(text, false);
 	LatexTables::removeColumn(ed->document(),row,col);
-
-        QEQUAL(ed->document()->text(),newText);
-
+	
+	QEQUAL(ed->document()->text(),newText);
+	
 }
 
 void TableManipulationTest::remRow_data(){
@@ -231,7 +274,7 @@ void TableManipulationTest::remRow(){
 	QFETCH(int, col);
 	QFETCH(QString, newText);
 
-	ed->setText(text);
+	ed->setText(text, false);
 	ed->setCursorPosition(row,col);
 	QDocumentCursor c(ed->cursor());
 	LatexTables::removeRow(c);
@@ -334,7 +377,7 @@ void TableManipulationTest::getCol(){
 	QFETCH(int, col);
 	QFETCH(int, colFound);
 
-	ed->setText(text);
+	ed->setText(text, false);
 	ed->setCursorPosition(row,col);
 	QDocumentCursor c(ed->cursor());
 	int nc=LatexTables::getColumn(c);
@@ -412,7 +455,7 @@ void TableManipulationTest::getNumberOfCol(){
 	QFETCH(int, col);
 	QFETCH(int, colFound);
 
-	ed->setText(text);
+	ed->setText(text, false);
 	ed->setCursorPosition(row,col);
 	QDocumentCursor c(ed->cursor());
 	int nc=LatexTables::getNumberOfColumns(c);
@@ -464,7 +507,7 @@ void TableManipulationTest::findNextToken(){
 	QFETCH(int, newCol);
 
 
-	ed->setText(text);
+	ed->setText(text, false);
 	ed->setCursorPosition(row,col);
 	QDocumentCursor c(ed->cursor());
 	QStringList tokens;
@@ -520,7 +563,7 @@ void TableManipulationTest::findNextTokenBackwards(){
 	QFETCH(int, newCol);
 
 
-	ed->setText(text);
+	ed->setText(text, false);
 	ed->setCursorPosition(row,col);
 	QDocumentCursor c(ed->cursor());
 	QStringList tokens;
@@ -581,7 +624,7 @@ void TableManipulationTest::addHLine(){
 	QFETCH(bool, remove);
 	QFETCH(QString, newText);
 
-	ed->setText(text);
+	ed->setText(text, false);
 	ed->setCursorPosition(row,col);
 	QDocumentCursor c(ed->cursor());
 	LatexTables::addHLine(c,numberOfLines,remove);
@@ -706,7 +749,7 @@ void TableManipulationTest::getDef(){
 	QFETCH(int, col);
 	QFETCH(QString, def_soll);
 
-	ed->setText(text);
+	ed->setText(text, false);
 	ed->setCursorPosition(row,col);
 	QDocumentCursor c(ed->cursor());
 	QString def=LatexTables::getDef(c);
@@ -717,5 +760,53 @@ void TableManipulationTest::getDef(){
 
 }
 
+void TableManipulationTest::parseTableLine_data(){
+	QTest::addColumn<QString>("text");
+	QTest::addColumn<int>("startCol");
+	QTest::addColumn<int>("behindCol");
+	QTest::addColumn<QString>("metaCommands");
+	QTest::addColumn<QString>("lineText");
+	QTest::addColumn<QString>("lineBreakOption");
+
+	QTest::newRow("simple")
+		<< "Row1 \\\\ Row2"
+		<< 0 << 7
+		<< "" << "Row1" << "";
+	QTest::newRow("simple2")
+		<< "Row1 \\\\ Row2"
+		<< 7 << 12
+		<< "" << "Row2" << "";
+	QTest::newRow("simple3")
+		<< "Row 1 & Col 2 \\\\ Row 2 & Col 2"
+		<< 0 << 16
+		<< "" << "Row 1 & Col 2" << "";
+	QTest::newRow("lineBreakOption")
+		<< "Row1 \\\\[1ex] Row2"
+		<< 0 << 12
+		<< "" << "Row1" << "[1ex]";
+	QTest::newRow("metaCommand")
+		<< "\\hline Row1 \\\\ Row2"
+		<< 0 << 14
+		<< "\\hline" << "Row1" << "";
+}
+
+void TableManipulationTest::parseTableLine(){
+	QFETCH(QString, text);
+	QFETCH(int, startCol);
+	QFETCH(int, behindCol);
+	QFETCH(QString, metaCommands);
+	QFETCH(QString, lineText);
+	QFETCH(QString, lineBreakOption);
+
+	LatexTableModel model(this);
+	LatexTableLine *ltl = model.parseNextLine(text, startCol);
+	if (ltl) {
+		QEQUAL(startCol, behindCol);
+		QEQUAL(ltl->toMetaLine(), metaCommands);
+		QEQUAL(ltl->toColLine(), lineText);
+		QEQUAL(ltl->toLineBreakOption(), lineBreakOption);
+	}
+}
+
 #endif
 
