@@ -20,21 +20,23 @@
 #include "buildmanager.h"
 #include "codesnippet.h"
 #include "configmanager.h"
+#include "cursorhistory.h"
 #include "latexlog.h"
 #include "latexdocument.h"
 #include "latexeditorview.h"
 #include "latexcompleter.h"
-#include "templateselector.h"
 //#include "symbollistwidget.h"
 #include "symbolgridwidget.h"
 #include "xmltagslistwidget.h"
 #include "spellerdialog.h"
 #include "textanalysis.h"
 #include "toolwidgets.h"
-#include "tmxtabwidget.h"
+#include "txstabwidget.h"
 #include "unicodeinsertion.h"
 #include "tablemanipulation.h"
 #include "PDFDocument.h"
+#include <QSplitter>
+#include <QSplashScreen>
 
 #include "qformatfactory.h"
 #include "qlanguagefactory.h"
@@ -53,50 +55,43 @@ enum SVNSTATUS {
 	InConflict
 };
 
-enum RunCommandFlag{
-	RCF_SHOW_STDOUT = 1,//low priority, overriden by configmanager always, both overriden by /dev/null redirection
-	RCF_WAIT_FOR_FINISHED = 2,
-	RCF_VIEW_LOG = 4, //show the log (only use if the command changes the log i.e. is latex), also enable reruns
-	RCF_SINGLE_INSTANCE = 8, //single viewer only
-	RCF_CHANGE_PDF = 16,
-	RCF_NO_DOCUMENT = 32, // don't check if document is saved as it is not used
-	RCF_IS_RERUN_CALL = 64 // call is an automatically rerun (internal)
-};
-Q_DECLARE_FLAGS(RunCommandFlags, RunCommandFlag);
-
-
-
-
-
 class UserMenuDialog;
+class GrammarCheck;
 class Texmaker : public QMainWindow {
 	Q_OBJECT
 	
 public:
-	Texmaker(QWidget *parent = 0, Qt::WFlags flags = 0);
+	Texmaker(QWidget *parent = 0, Qt::WFlags flags = 0, QSplashScreen *splash = 0);
 	~Texmaker();
 	
 	Q_INVOKABLE QString getCurrentFileName(); //returns the absolute file name of the current file or "" if none is opene
 	Q_INVOKABLE QString getAbsoluteFilePath(const QString & relName, const QString &extension=""); //treats the path relative to the compiled .tex file
+    Q_INVOKABLE QString getRelativeFileName(const QString & file,QString basepath); //provide function for scripts
 	QByteArray windowstate;
 	bool tobemaximized,tobefullscreen;
 	
 public slots:
-	LatexEditorView* load(const QString &f , bool asProject = false);
+    LatexEditorView* load(const QString &f , bool asProject = false, bool hidden = false);
 	void executeCommandLine(const QStringList& args, bool realCmdLine);
 	void onOtherInstanceMessage(const QString &);  // For messages for the single instance
 	
-private:
+	void fuzzCursorHistory();
+	void fuzzBackForward();
+
+
+protected:
 	//these are just wrappers around configManager so we don't have to type so much (todo??? move them to configmanager.h and use a singleton design???)
-	inline QMenu* newManagedMenu(const QString &id,const QString &text);
-	inline QMenu* newManagedMenu(QMenu* menu, const QString &id,const QString &text);
-	inline QAction* newManagedAction(QWidget* menu, const QString &id,const QString &text, const char* slotName=0, const QKeySequence &shortCut=0, const QString & iconFile="", const QList<QVariant>& args = QList<QVariant>());
-	inline QAction* newManagedAction(QWidget* menu, const QString &id,const QString &text, const char* slotName, const QList<QKeySequence> &shortCuts, const QString & iconFile="", const QList<QVariant>& args = QList<QVariant>());
-	inline QAction* newManagedEditorAction(QWidget* menu, const QString &id,const QString &text, const char* slotName=0, const QKeySequence &shortCut=0, const QString & iconFile="", const QList<QVariant>& args = QList<QVariant>());
-	inline QAction* newManagedEditorAction(QWidget* menu, const QString &id,const QString &text, const char* slotName, const QList<QKeySequence> &shortCuts, const QString & iconFile="", const QList<QVariant>& args = QList<QVariant>());
-	inline QAction* newManagedAction(QWidget* menu, const QString &id, QAction* act);
-	inline QAction* getManagedAction(QString id);
-	QAction* insertManagedAction(QAction* before, const QString &id,const QString &text, const char* slotName=0, const QKeySequence &shortCut=0, const QString & iconFile="");
+	Q_INVOKABLE inline QMenu* newManagedMenu(const QString &id,const QString &text){ return configManager.newManagedMenu(id,text); }
+	Q_INVOKABLE inline QMenu* newManagedMenu(QMenu* menu, const QString &id,const QString &text){ return configManager.newManagedMenu(menu,id,text); }
+	Q_INVOKABLE QAction* newManagedAction(QWidget* menu, const QString &id,const QString &text, const char* slotName=0, const QKeySequence &shortCut=0, const QString & iconFile="", const QList<QVariant>& args = QList<QVariant>());
+	Q_INVOKABLE QAction* newManagedAction(QWidget* menu, const QString &id,const QString &text, const char* slotName, const QList<QKeySequence> &shortCuts, const QString & iconFile="", const QList<QVariant>& args = QList<QVariant>());
+	Q_INVOKABLE QAction* newManagedEditorAction(QWidget* menu, const QString &id,const QString &text, const char* slotName=0, const QKeySequence &shortCut=0, const QString & iconFile="", const QList<QVariant>& args = QList<QVariant>());
+	Q_INVOKABLE QAction* newManagedEditorAction(QWidget* menu, const QString &id,const QString &text, const char* slotName, const QList<QKeySequence> &shortCuts, const QString & iconFile="", const QList<QVariant>& args = QList<QVariant>());
+	Q_INVOKABLE inline QAction* newManagedAction(QWidget* menu, const QString &id, QAction* act) { return configManager.newManagedAction(menu,id,act); }
+	Q_INVOKABLE inline QMenu* getManagedMenu(const QString& id){ return configManager.getManagedMenu(id); }
+	Q_INVOKABLE inline QAction* getManagedAction(const QString& id){ return configManager.getManagedAction(id); }
+	Q_INVOKABLE inline QList<QAction *> getManagedActions(const QStringList& ids, const QString &commonPrefix=""){ return configManager.getManagedActions(ids, commonPrefix); }
+	Q_INVOKABLE QAction* insertManagedAction(QAction* before, const QString &id,const QString &text, const char* slotName=0, const QKeySequence &shortCut=0, const QString & iconFile="");
 	
 	SymbolGridWidget* addSymbolGrid(const QString& SymbolList,  const QString& iconName, const QString& text);
 	void addTagList(const QString& id, const QString& iconName, const QString& text, const QString& tagFile);
@@ -106,36 +101,48 @@ private:
 private slots:
 	void updateToolBarMenu(const QString& menuName);
 private:
+	void executeTests(const QStringList &args);
+	void generateAddtionalTranslations();
 	void setupMenus();
 	void setupDockWidgets();
 	void setupToolBars();
 	void createStatusBar();
-	bool FileAlreadyOpen(QString f, bool checkTemporaryNames = false);
+	bool FocusEditorForFile(QString f, bool checkTemporaryNames = false);
 	bool closeAllFilesAsking(); //asks the user and close all files
 	bool canCloseNow(); //asks the user and close all files, and prepares to exit txs
 	void closeEvent(QCloseEvent *e);
 	
-	QFormatFactory *m_formats;
+	void updateUserMacros(bool updateMenu = true);
+	
+	QFormatScheme *m_formats, *m_formatsOldDefault;
 	QLanguageFactory* m_languages;
 	LatexCompleter* completer;
 	
 	QPointer<UnicodeInsertion> unicodeInsertionDialog;
 	
 	//gui
-	TmxTabWidget *EditorView;
+	TxsTabWidget *EditorView;
+	QSplitter *splitter;
 	QToolBar* centralToolBar;
 	CustomWidgetList *leftPanel;
 	SymbolGridWidget *MostUsedSymbolWidget, *FavoriteSymbolWidget;
 	QString hiddenLeftPanelWidgets;
 	
 	QTreeView *structureTreeView;
+	QListWidget *bookmarksWidget;
 	LatexParser latexParser;
+public:
 	LatexDocuments documents;
-	
+private:
 	OutputViewWidget *outputView; //contains output widgets (over OutputLayout)
 	
+	//menu
+	QActionGroup *bibtexEntryActions;
+	QActionGroup *biblatexEntryActions;
+	QActionGroup *bibTypeActions;
+	QActionGroup *highlightLanguageActions;
+
 	//toolbars
-	//
 	QAction *ToggleAct, *ToggleRememberAct;
 	QAction *actSave, *actUndo, *actRedo;
 	
@@ -143,51 +150,53 @@ private:
 	QToolButton *statusTbLanguage;
 	QActionGroup *spellLanguageActions;
 	
-	QToolButton *combo1,*combo2,*combo;
-	
+	CursorHistory *cursorHistory;
+
 	//settings
 	ConfigManager configManager;
+public:
 	BuildManager buildManager;
-	int split1_right, split1_left, split2_top, split2_bottom;
-	QString spell_ignored_words;
+private:
 	QStringList struct_level;
-	QStringList labelitem;
 	
-	QStringList userTemplatesList;
+	qreal pdfSplitterRel;
 	
 	SpellerManager spellerManager;
+	SafeThread grammarCheckThread;
+	GrammarCheck *grammarCheck;
 	
 	//dialogs
 	TextAnalysisDialog *textAnalysisDlg;
 	SpellerDialog *spellDlg;
-	templateselector *templateSelectorDialog;
 	
 	//tools
-	bool FINPROCESS, ERRPROCESS, KILLPROCESS, PROCESSRUNNING;
-	
-	SymbolList symbolScore;
 	usercodelist symbolMostused;
 	QStringList symbolFavorites;
 	
 	Q_INVOKABLE LatexEditorView *currentEditorView() const;
 	Q_INVOKABLE QEditor* currentEditor() const;
 	void configureNewEditorView(LatexEditorView *edit);
-	void configureNewEditorViewEnd(LatexEditorView *edit,bool asMaster=false);
+    void configureNewEditorViewEnd(LatexEditorView *edit,bool asMaster=false, bool hidden = false);
 	LatexEditorView* getEditorViewFromFileName(const QString &fileName, bool checkTemporaryNames = false);
 	
-	QAction* outputViewAction, *fullscreenModeAction;
+	QAction *fullscreenModeAction;
 	
+	int runningPDFCommands, runningPDFAsyncCommands;
+
+	void updateUserToolMenu();
 	void linkToEditorSlot(QAction* act, const char* slot, const QList<QVariant>& args);
 private slots:
 	void relayToEditorSlot();
 	void relayToOwnSlot();
+	void autoRunScripts();
+	void runScripts(int trigger);
 	
 protected slots:
 	void fileNew(QString fileName="");
 	void fileNewFromTemplate();
 	void fileMakeTemplate();
 	void fileOpen();
-	void fileRestoreSession();
+	void fileRestoreSession(bool showProgress = true);
 	void fileSave();
 	void fileSaveAll();
 	void fileSaveAll(bool alsoUnnamedFiles, bool alwaysCurrentFile);
@@ -252,8 +261,12 @@ protected slots:
 	void editHardLineBreak();
 	void editHardLineBreakRepeat();
 	void editEraseLine();
+	void editEraseEndLine();
+	void editMoveLineUp();
+	void editMoveLineDown();
+	void editDuplicateLine();
 	void editEraseWordCmdEnv();
-	void editGotoDefinition();
+	void editGotoDefinition(QDocumentCursor c = QDocumentCursor());
 	void editSpell();
 	void editThesaurus(int line=-1,int col=-1);
 	void editChangeLineEnding();
@@ -275,12 +288,19 @@ protected slots:
 	void symbolRemoveFavorite();
 	void symbolRemoveAllFavorites();
 	
+	void editorTabContextMenu(const QPoint& point);
+
+    void addDocToLoad(QString filename);
+    void removeFromEditor(LatexEditorView *edView);
+
+    void moveCursorTodlh();
+
 private slots:	
 	void ReadSettings();
 	void SaveSettings(const QString& configName="");
 	
 protected slots:	
-	void showMarkTooltipForLogMessage(int error);
+	void showMarkTooltipForLogMessage(QList<int> errors);
 	void NewDocumentStatus();
 	void NewDocumentLineEnding();
 	void UpdateCaption();
@@ -290,15 +310,24 @@ protected slots:
 	void CloseEditorTab(int tab);
 	void EditorTabMoved(int from,int to);
 	
-	void updateStructure(bool initial=false);
+    void updateStructure(bool initial=false, LatexDocument *doc = 0);
 	void ShowStructure();
 	void clickedOnStructureEntry(const QModelIndex & index);
+    void clickedOnBookmark(QListWidgetItem *item);
 	void editRemovePlaceHolders();
+    void moveBookmarkUp();
+    void moveBookmarkDown();
+    void removeBookmark();
+    void removeAllBookmarks();
+    void updateLineWithBookmark(int lineNr);
+    void updateBookmarks(LatexEditorView *edView);
 	
 	void NormalCompletion();
 	void InsertEnvironmentCompletion();
 	void InsertTextCompletion();
-	void InsertTag(QString Entity, int dx, int dy);
+	void InsertTag(const QString &Entity, int dx=0, int dy=0);
+	void InsertCitation(const QString &text);
+	void InsertFormula(const QString &formula);
 	void InsertSymbol(QTableWidgetItem *item);
 	void InsertSymbolPressed(QTableWidgetItem *item);
 	void InsertXmlTag(QListWidgetItem *item);	
@@ -310,15 +339,19 @@ protected slots:
 	
 	void InsertBibEntryFromAction();
 	void InsertBibEntry(const QString& id="");
+	void SetBibTypeFromAction();
 	
-	void insertUserTag(const QString& macro);
+	void insertUserTag(const QString& macro, int triggerId = 0);
 	void insertUserTag();
-	void EditUserMenu();
-	void userMacroDialogAccepted();
+	void editMacros();
+	void macroDialogAccepted();
 	
+	void InsertRef(const QString &refCmd);
 	void InsertRef();
+	void InsertEqRef();
 	void InsertPageRef();
-	
+	void createLabelFromAction();
+
 	void UpdateAvailableLanguages();
 	void EditorSpellerChanged(const QString &name);
 	void ChangeEditorSpeller();
@@ -330,22 +363,25 @@ protected slots:
 	void QuickLetter();
 	void QuickDocument();
 	void QuickGraphics(const QString &graphicsFile = QString());
+	void QuickMath();
 	
+	bool checkProgramPermission(const QString& program, const QString& cmdId, LatexDocument* master);
+	void runInternalPdfViewer(const QFileInfo& master, const QString& options);
+	void runBibliographyIfNecessary(const QFileInfo& cmd);
 private slots:	
-	void runCommand(BuildManager::LatexCommand cmd, RunCommandFlags flags);
-	void runCommand(const QString& commandline, RunCommandFlags flags = 0, QString *buffer = 0); //default flags == 0 is necessary for pdf viewer
-	void RunPreCompileCommand();
-	void readFromStderr();
-	void readFromStdoutput();
-	void SlotEndProcess(int err);
-	void runCommandList(const QStringList& commandList, const RunCommandFlags& additionalFlags);
+	void runInternalCommand(const QString& cmd, const QFileInfo& master, const QString& options);
+	void commandLineRequested(const QString& cmdId, QString* result, bool*);
+	void beginRunningCommand(const QString& commandMain, bool latex, bool pdf, bool async);
+	void beginRunningSubCommand(ProcessX* p, const QString& commandMain, const QString& subCommand, const RunCommandFlags& flags);
+	void endRunningSubCommand(ProcessX* p, const QString& commandMain, const QString& subCommand, const RunCommandFlags& flags);
+	void endRunningCommand(const QString& commandMain, bool latex, bool pdf, bool async);
+	
+	
+	bool runCommand(const QString& commandline, QString* buffer = 0);
 protected slots:	
 	void processNotification(const QString& message);
-	void QuickBuild();
 	void CleanAll();
 	void commandFromAction();  //calls a command given by sender.data, doesn't wait
-	void UserTool();
-	void EditUserTool();
 	
 	void WebPublish();
 	void WebPublishSource();
@@ -355,7 +391,7 @@ protected slots:
 	
 	void RealViewLog(bool noTabChange=false);
 	void ViewLog(bool noTabChange=false);
-	void ViewLogOrReRun();
+	void ViewLogOrReRun(LatexCompileResult* result);
 	void DisplayLatexError();
 	bool gotoNearLogEntry(int lt, bool backward, QString notFoundMessage);
 	bool NoLatexErrors();
@@ -374,55 +410,70 @@ protected slots:
 	void gotoOpenDocument();
 	void updateOpenDocumentMenu(bool localChange=false);
 	
-	void viewToggleOutputView();
+	void focusEditor();
+	void focusViewer();
+
 	void viewCloseSomething();
 	void setFullScreenMode();
-	void viewSetHighlighting();
+	void viewAlignWindows();
+	void viewSetHighlighting(QAction *act);
+	void showHighlightingMenu();
 	
 	void viewCollapseBlock();
 	void viewExpandBlock();
 	
-	void newPdfPreviewer();
+	QObject* newPdfPreviewer(bool embedded=false);
+	void pdfClosed();
+	void restoreMacMenuBar();
 	
 	void masterDocumentChanged(LatexDocument * doc);
+	void aboutToDeleteDocument(LatexDocument * doc);
 	
 	void SetMostUsedSymbols(QTableWidgetItem* item);
 	
-	void updateCompleter();
+    void updateCompleter(LatexEditorView *edView = 0);
 	void completerNeedsUpdate();
 	void needUpdatedCompleter();
 	
 	void tabChanged(int i);
 	
-	void gotoLine(int line);  //0 based
+	void gotoLine(int line, int col=0, LatexEditorView *edView = 0);  // line is 0 based
 	void gotoLocation(int line, const QString &fileName);  //0 based, absolute file name
 	void gotoLogEntryEditorOnly(int logEntryNumber);
 	bool gotoLogEntryAt(int newLineNumber);
 	bool gotoMark(bool backward, int id=-1);
 	void syncFromViewer(const QString &fileName, int line, bool activate, const QString& guessedWord);  //0 based, absolute file name, raise window
 	
+	void goBack();
+	void goForward();
+	void setGlobalCursor(const QDocumentCursor &c);
+	void saveCurrentCursorToHistory();
+
 	void previewLatex();
-	void previewAvailable(const QString& imageFile, const QString& text, int line);
+	void previewAvailable(const QString& imageFile, const PreviewSource& source);
 	void clearPreview();
 	void showPreview(const QString& text);
 	void showPreview(const QDocumentCursor& c);
 	void showPreview(const QDocumentCursor& c, bool addToList);
 	void showPreviewQueue();
+    void showImgPreview(const QString& fname);
+    void showImgPreviewFinished(const QPixmap& pm, int page);
 	
-	void templateRemove();
-	void templateEdit();
+	void templateEdit(const QString &fname);
 	
 	void fileReloaded();
 	void fileInConflict();
 	void fileAutoReloading(QString fname);
 	
-	void jumpToSearch(QString filename,int lineNumber);
+	void jumpToSearch(QDocument* doc,int lineNumber);
 	
 	void cursorPositionChanged();
+	void syncPDFViewer(QDocumentCursor cur, bool inForeground = true);
 	//	void treeWidgetChanged();
 	
 	bool generateMirror(bool setCur=false);
 	void generateBracketInverterMirror();
+	void jumpToBracket();
 	void selectBracket();
 	void findMissingBracket();
 	void cursorHovered();
@@ -438,6 +489,7 @@ protected slots:
 	void addHLineCB();
 	void remHLineCB();
 	void insertTableTemplate();
+    void alignTableCols();
 	
 	void latexModelViewMode();
 	void moveDocumentToFront();
@@ -446,6 +498,10 @@ protected slots:
 	
 	void updateHighlighting();
 	
+	void toggleGrammar(int type);
+
+    void bookmarkDeleted(QDocumentLineHandle* dlh);
+    void bookmarkAdded(QDocumentLineHandle* dlh,int nr);
 private slots:
 	void importPackage(QString name);
 	void packageScanCompleted(QString name);
@@ -483,28 +539,50 @@ protected:
 	QString fileFilters;
 	QString selectedFileFilter;
 	
+	bool runBibliographyIfNecessaryEntered;
 	
 	LatexStyleParser *latexStyleParser;
-	
-	int remainingReRunCount;
-	bool lastReRunWasBibTeX;
-	QString rerunCommand;
-	RunCommandFlags rerunFlags;
-	
+		
 	QMap<QString,QString> detectedEnvironmentsForHighlighting;
 	
 	LatexDocument* diffLoadDocHidden(QString f);
 	void showDiff3(const QString file1,const QString file2);
 	
+	void restoreBookmarks(LatexEditorView *edView);
 	
+    bool completerPreview;
 	
+	LatexEditorView *editorViewForLabel(LatexDocument *doc, const QString &label);
+
 public:
 	Q_PROPERTY(QString clipboard READ clipboardText WRITE setClipboardText);
 	Q_INVOKABLE QString clipboardText(const QClipboard::Mode& mode = QClipboard::Clipboard) const;
 	Q_INVOKABLE void setClipboardText(const QString& text, const QClipboard::Mode& mode = QClipboard::Clipboard);
 	Q_INVOKABLE int getVersion() const;
+	
+	static void recoverFromCrash();
+	
+	
+public slots:
+	void threadCrashed();
+	void iamalive();
+    void checkLatexInstall();
+	
+	void slowOperationStarted();
+	void slowOperationEnded();
+	
+	
+signals:
+	void infoNewFile();
+	void infoNewFromTemplate();
+	void infoLoadFile(const QString& filename);
+	void infoFileSaved(const QString& filename);
+	void infoFileClosed();
+	void infoAfterTypeset();
+    void ImgPreview(const QString& fn);
 };
 
 Q_DECLARE_METATYPE(Texmaker*)
 
 #endif
+
